@@ -22,6 +22,7 @@
 #include <gnunet/gnunet_util_lib.h>
 #include <jansson.h>
 #include <pthread.h>
+#include "taler_merchant_util.h"
 #include "taler_merchantdb_lib.h"
 #include "taler_merchantdb_plugin.h"
 #include <taler/taler_dbevents.h>
@@ -176,11 +177,6 @@ static struct Child *c_tail;
  */
 static struct TALER_EXCHANGE_Keys *keys;
 
-/**
- * Handle for active /keys request.
- */
-static struct TALER_EXCHANGE_GetKeysHandle *gkh;
-
 /**
  * Head of list of active exchange interactions.
  */
@@ -201,6 +197,11 @@ static uint64_t w_count;
  */
 static struct GNUNET_DB_EventHandler *eh;
 
+/**
+ * Notification handler from database on new keys.
+ */
+static struct GNUNET_DB_EventHandler *keys_eh;
+
 /**
  * The merchant's configuration.
  */
@@ -272,16 +273,16 @@ shutdown_task (void *cls)
     db_plugin->event_listen_cancel (eh);
     eh = NULL;
   }
+  if (NULL != keys_eh)
+  {
+    db_plugin->event_listen_cancel (keys_eh);
+    keys_eh = NULL;
+  }
   if (NULL != task)
   {
     GNUNET_SCHEDULER_cancel (task);
     task = NULL;
   }
-  if (NULL != gkh)
-  {
-    TALER_EXCHANGE_get_keys_cancel (gkh);
-    gkh = NULL;
-  }
   while (NULL != (w = w_head))
   {
     GNUNET_CONTAINER_DLL_remove (w_head,
@@ -414,6 +415,7 @@ deposit_get_cb (
   struct ExchangeInteraction *w = cls;
   struct GNUNET_TIME_Absolute future_retry;
 
+  w->dgh = NULL;
   future_retry
     = GNUNET_TIME_relative_to_absolute (w->retry_backoff);
   switch (dr->hr.http_status)
@@ -436,19 +438,6 @@ deposit_get_cb (
         GNUNET_SCHEDULER_shutdown ();
         return;
       }
-      qs = db_plugin->update_deposit_confirmation_status (
-        db_plugin->cls,
-        w->deposit_serial,
-        false, /* we are done, wire_pending is now false */
-        GNUNET_TIME_absolute_to_timestamp (future_retry),
-        w->retry_backoff,
-        NULL);
-      if (qs < 0)
-      {
-        GNUNET_break (0);
-        GNUNET_SCHEDULER_shutdown ();
-        return;
-      }
       break;
     }
   case MHD_HTTP_ACCEPTED:
@@ -463,19 +452,13 @@ deposit_get_cb (
                   dr->details.accepted.kyc_ok,
                   TALER_B2S (&w->coin_pub));
       now = GNUNET_TIME_timestamp_get ();
-      qs = db_plugin->account_kyc_set_status (
+      qs = db_plugin->account_kyc_set_failed (
         db_plugin->cls,
         w->instance_id,
         &w->h_wire,
         exchange_url,
-        dr->details.accepted.requirement_row,
         now,
         MHD_HTTP_ACCEPTED,
-        TALER_EC_NONE,
-        NULL,
-        0,
-        NULL,
-        false,
         dr->details.accepted.kyc_ok);
       if (qs < 0)
       {
@@ -755,42 +738,6 @@ select_work (void *cls)
 }
 
 
-/**
- * Function called with information about who is auditing
- * a particular exchange and what keys the exchange is using.
- * The ownership over the @a keys object is passed to
- * the callee, thus it is given explicitly and not
- * (only) via @a kr.
- *
- * @param cls closure, NULL
- * @param kr response from /keys
- * @param[in] in_keys keys object passed to callback with
- *  reference counter of 1. Must be freed by callee
- *  using TALER_EXCHANGE_keys_decref(). NULL on failure.
- */
-static void
-keys_cb (
-  void *cls,
-  const struct TALER_EXCHANGE_KeysResponse *kr,
-  struct TALER_EXCHANGE_Keys *in_keys)
-{
-  gkh = NULL;
-  if (NULL == in_keys)
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "Failed to download %skeys\n",
-                exchange_url);
-    GNUNET_SCHEDULER_shutdown ();
-    return;
-  }
-  keys = TALER_EXCHANGE_keys_incref (in_keys);
-  if (NULL != task)
-    GNUNET_SCHEDULER_cancel (task);
-  task = GNUNET_SCHEDULER_add_now (&select_work,
-                                   NULL);
-}
-
-
 /**
  * Start a copy of this process with the exchange URL
  * set to the given @a base_url
@@ -939,6 +886,76 @@ cfg_iter_cb (void *cls,
 }
 
 
+/**
+ * Trigger (re)loading of keys from DB.
+ *
+ * @param cls NULL
+ * @param extra base URL of the exchange that changed
+ * @param extra_len number of bytes in @a extra
+ */
+static void
+update_exchange_keys (void *cls,
+                      const void *extra,
+                      size_t extra_len)
+{
+  const char *url = extra;
+
+  if ( (NULL == extra) ||
+       (0 == extra_len) )
+  {
+    GNUNET_break (0);
+    return;
+  }
+  if ('\0' != url[extra_len - 1])
+  {
+    GNUNET_break (0);
+    return;
+  }
+  if (0 != strcmp (url,
+                   exchange_url))
+    return; /* not relevant for us */
+
+  {
+    enum GNUNET_DB_QueryStatus qs;
+
+    if (NULL != keys)
+    {
+      TALER_EXCHANGE_keys_decref (keys);
+      keys = NULL;
+    }
+    qs = db_plugin->select_exchange_keys (db_plugin->cls,
+                                          exchange_url,
+                                          &keys);
+    if (qs < 0)
+    {
+      GNUNET_break (0);
+      GNUNET_SCHEDULER_shutdown ();
+      return;
+    }
+    if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                  "No keys yet for `%s'\n",
+                  exchange_url);
+    }
+  }
+  if (NULL == keys)
+  {
+    if (NULL != task)
+    {
+      GNUNET_SCHEDULER_cancel (task);
+      task = NULL;
+    }
+  }
+  else
+  {
+    if (NULL == task)
+      task = GNUNET_SCHEDULER_add_now (&select_work,
+                                       NULL);
+  }
+}
+
+
 /**
  * First task.
  *
@@ -1001,7 +1018,7 @@ run (void *cls,
       db_plugin->connect (db_plugin->cls))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Failed to connect to database\n");
+                "Failed to connect to database. Consider running taler-merchant-dbinit!\n");
     GNUNET_SCHEDULER_shutdown ();
     global_ret = EXIT_NO_RESTART;
     return;
@@ -1018,11 +1035,22 @@ run (void *cls,
                                   &db_notify,
                                   NULL);
   }
-  gkh = TALER_EXCHANGE_get_keys (ctx,
-                                 exchange_url,
-                                 NULL,
-                                 &keys_cb,
-                                 NULL);
+  {
+    struct GNUNET_DB_EventHeaderP es = {
+      .size = ntohs (sizeof (es)),
+      .type = ntohs (TALER_DBEVENT_MERCHANT_EXCHANGE_KEYS)
+    };
+
+    keys_eh = db_plugin->event_listen (db_plugin->cls,
+                                       &es,
+                                       GNUNET_TIME_UNIT_FOREVER_REL,
+                                       &update_exchange_keys,
+                                       NULL);
+  }
+
+  update_exchange_keys (NULL,
+                        exchange_url,
+                        strlen (exchange_url) + 1);
 }
 
 
@@ -1054,19 +1082,14 @@ main (int argc,
   };
   enum GNUNET_GenericReturnValue ret;
 
-  if (GNUNET_OK !=
-      GNUNET_STRINGS_get_utf8_args (argc, argv,
-                                    &argc, &argv))
-    return EXIT_INVALIDARGUMENT;
-  TALER_OS_init ();
   ret = GNUNET_PROGRAM_run (
+    TALER_MERCHANT_project_data (),
     argc, argv,
     "taler-merchant-depositcheck",
     gettext_noop (
       "background process that checks with the exchange on deposits that are past the wire deadline"),
     options,
     &run, NULL);
-  GNUNET_free_nz ((void *) argv);
   if (GNUNET_SYSERR == ret)
     return EXIT_INVALIDARGUMENT;
   if (GNUNET_NO == ret)
