@@ -28,6 +28,7 @@
 #include <taler/taler_mhd_lib.h>
 #include <taler/taler_templating_lib.h>
 #include <taler/taler_exchange_service.h>
+#include "taler_merchant_util.h"
 #include "taler-merchant-httpd_config.h"
 #include "taler-merchant-httpd_exchanges.h"
 #include "taler-merchant-httpd_get-orders-ID.h"
@@ -337,7 +338,7 @@ TMH_compute_auth (const char *token,
 void
 TMH_wire_method_free (struct TMH_WireMethod *wm)
 {
-  GNUNET_free (wm->payto_uri);
+  GNUNET_free (wm->payto_uri.full_payto);
   GNUNET_free (wm->wire_method);
   GNUNET_free (wm->credit_facade_url);
   json_decref (wm->credit_facade_credentials);
@@ -504,6 +505,7 @@ handle_mhd_completion_callback (void *cls,
     json_decref (hc->request_body);
   if (NULL != hc->instance)
     TMH_instance_decref (hc->instance);
+  TMH_db->preflight (TMH_db->cls);
   GNUNET_free (hc->full_url);
   GNUNET_free (hc);
   *con_cls = NULL;
@@ -1614,7 +1616,9 @@ url_handler (void *cls,
         if (MHD_NO ==
             MHD_add_response_header (response,
                                      MHD_HTTP_HEADER_LOCATION,
-                                     rslash))
+                                     NULL == rslash
+                                     ? "/"
+                                     : rslash))
         {
           GNUNET_break (0);
           MHD_destroy_response (response);
@@ -2011,20 +2015,25 @@ url_handler (void *cls,
  * Callback invoked with information about a bank account.
  *
  * @param cls closure with a `struct TMH_MerchantInstance *`
+ * @param merchant_priv private key of the merchant instance
  * @param acc details about the account
  */
 static void
 add_account_cb (void *cls,
+                const struct TALER_MerchantPrivateKeyP *merchant_priv,
                 const struct TALER_MERCHANTDB_AccountDetails *acc)
 {
   struct TMH_MerchantInstance *mi = cls;
   struct TMH_WireMethod *wm;
 
+  (void) merchant_priv;
   wm = GNUNET_new (struct TMH_WireMethod);
   wm->h_wire = acc->h_wire;
-  wm->payto_uri = GNUNET_strdup (acc->payto_uri);
+  wm->payto_uri.full_payto
+    = GNUNET_strdup (acc->payto_uri.full_payto);
   wm->wire_salt = acc->salt;
-  wm->wire_method = TALER_payto_get_method (acc->payto_uri);
+  wm->wire_method
+    = TALER_payto_get_method (acc->payto_uri.full_payto);
   wm->active = acc->active;
   GNUNET_CONTAINER_DLL_insert (mi->wm_head,
                                mi->wm_tail,
@@ -2257,17 +2266,13 @@ run (void *cls,
 
   if (GNUNET_SYSERR ==
       TALER_config_get_currency (cfg,
+                                 "merchant",
                                  &TMH_currency))
   {
 
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
-
-  /* TODO: Load config variables for merchant token family
-           cipher type "rsa" or "cs" and key size.
-           Defaults to "rsa" and 2048 bits. */
-
   if (GNUNET_OK !=
       TALER_CONFIG_parse_currencies (cfg,
                                      TMH_currency,
@@ -2311,7 +2316,14 @@ run (void *cls,
                                             "merchant",
                                             "FORCE_AUDIT"))
     TMH_force_audit = GNUNET_YES;
-  TALER_TEMPLATING_init ("merchant");
+  if (GNUNET_OK !=
+      TALER_TEMPLATING_init (TALER_MERCHANT_project_data ()))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Failed to setup templates\n");
+    GNUNET_SCHEDULER_shutdown ();
+    return;
+  }
   if (GNUNET_OK !=
       TMH_spa_init ())
   {
@@ -2339,7 +2351,7 @@ run (void *cls,
       TMH_db->connect (TMH_db->cls))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Failed to initialize database connection\n");
+                "Failed to connect to database. Consider running taler-merchant-dbinit!\n");
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
@@ -2439,12 +2451,13 @@ main (int argc,
     GNUNET_GETOPT_OPTION_END
   };
 
-  TALER_OS_init ();
-  res = GNUNET_PROGRAM_run (argc, argv,
-                            "taler-merchant-httpd",
-                            "Taler merchant's HTTP backend interface",
-                            options,
-                            &run, NULL);
+  res = GNUNET_PROGRAM_run (
+    TALER_MERCHANT_project_data (),
+    argc, argv,
+    "taler-merchant-httpd",
+    "Taler merchant's HTTP backend interface",
+    options,
+    &run, NULL);
   if (GNUNET_SYSERR == res)
     return EXIT_INVALIDARGUMENT;
   if (GNUNET_NO == res)
