@@ -31,9 +31,9 @@
  * Shorthand for exit jumps.
  */
 #define EXITIF(cond)                                              \
-  do {                                                            \
-    if (cond) { GNUNET_break (0); goto EXITIF_exit; }             \
-  } while (0)
+        do {                                                            \
+          if (cond) { GNUNET_break (0); goto EXITIF_exit; }             \
+        } while (0)
 
 
 /**
@@ -59,7 +59,7 @@ struct TMH_WireMethod
   /**
    * Wire details for this instance
    */
-  char *payto_uri;
+  struct TALER_FullPayto payto_uri;
 
   /**
    * Salt to use when computing @e h_wire from @e payto_uri.
