@@ -22,16 +22,19 @@
 #include "platform.h"
 #include <jansson.h>
 #include <gnunet/gnunet_uri_lib.h>
+#include <gnunet/gnunet_common.h>
 #include <taler/taler_signatures.h>
 #include <taler/taler_dbevents.h>
 #include <taler/taler_json_lib.h>
 #include <taler/taler_templating_lib.h>
 #include <taler/taler_exchange_service.h>
-#include "taler-merchant-httpd_exchanges.h"
 #include "taler-merchant-httpd_helper.h"
 #include "taler-merchant-httpd_get-orders-ID.h"
 #include "taler-merchant-httpd_mhd.h"
 #include "taler-merchant-httpd_qr.h"
+#include "taler/taler_error_codes.h"
+#include "taler/taler_util.h"
+#include "taler_merchant_util.h"
 
 /**
  * How often do we retry DB transactions on serialization failures?
@@ -96,15 +99,22 @@ struct GetOrderData
   struct TMH_SuspendedConnection sc;
 
   /**
-   * Database event we are waiting on to be resuming.
+   * Database event we are waiting on to be resuming on payment.
    */
   struct GNUNET_DB_EventHandler *pay_eh;
 
   /**
-   * Database event we are waiting on to be resuming.
+   * Database event we are waiting on to be resuming for refunds.
    */
   struct GNUNET_DB_EventHandler *refund_eh;
 
+  /**
+   * Database event we are waiting on to be resuming for repurchase
+   * detection updating some equivalent order (same fulfillment URL)
+   * to our session.
+   */
+  struct GNUNET_DB_EventHandler *session_eh;
+
   /**
    * Which merchant instance is this for?
    */
@@ -115,44 +125,26 @@ struct GetOrderData
    */
   const char *order_id;
 
-  /**
-   * Where to get the contract
-   */
-  const char *contract_url;
-
-  /**
-   * fulfillment URL of the contract (valid as long as @e contract_terms is
-   * valid; but can also be NULL if the contract_terms does not come with
-   * a fulfillment URL).
-   */
-  const char *fulfillment_url;
-
   /**
    * session of the client
    */
   const char *session_id;
 
+  /**
+   * choice index (contract v1)
+   */
+  int16_t choice_index;
+
   /**
    * Contract terms of the payment we are checking. NULL when they
    * are not (yet) known.
    */
-  json_t *contract_terms;
+  json_t *contract_terms_json;
 
   /**
-   * Merchant base URL from @e contract_terms.
+   * Parsed contract terms, NULL when parsing failed.
    */
-  const char *merchant_base_url;
-
-  /**
-   * Public reorder URL from @e contract_terms.
-   * Could be NULL if contract does not have one.
-   */
-  const char *public_reorder_url;
-
-  /**
-   * Total amount in contract.
-   */
-  struct TALER_Amount contract_total;
+  struct TALER_MERCHANT_Contract *contract_terms;
 
   /**
    * Total refunds granted for this payment. Only initialized
@@ -307,14 +299,19 @@ suspend_god (struct GetOrderData *god)
   /* We reset the contract terms and start by looking them up
      again, as while we are suspended fundamental things could
      change (such as the contract being claimed) */
+  if (NULL != god->contract_terms_json)
+  {
+    json_decref (god->contract_terms_json);
+    god->contract_terms->fulfillment_url = NULL;
+    god->contract_terms_json = NULL;
+    god->contract_parsed = false;
+    god->contract_terms->merchant_base_url = NULL;
+    god->contract_terms->public_reorder_url = NULL;
+  }
   if (NULL != god->contract_terms)
   {
-    json_decref (god->contract_terms);
-    god->fulfillment_url = NULL;
+    TALER_MERCHANT_contract_free (god->contract_terms);
     god->contract_terms = NULL;
-    god->contract_parsed = false;
-    god->merchant_base_url = NULL;
-    god->public_reorder_url = NULL;
   }
   GNUNET_assert (! god->suspended);
   god->contract_parsed = false;
@@ -340,11 +337,21 @@ god_cleanup (void *cls)
 {
   struct GetOrderData *god = cls;
 
+  if (NULL != god->contract_terms_json)
+  {
+    json_decref (god->contract_terms_json);
+    god->contract_terms_json = NULL;
+  }
   if (NULL != god->contract_terms)
   {
-    json_decref (god->contract_terms);
+    TALER_MERCHANT_contract_free (god->contract_terms);
     god->contract_terms = NULL;
   }
+  if (NULL != god->session_eh)
+  {
+    TMH_db->event_listen_cancel (god->session_eh);
+    god->session_eh = NULL;
+  }
   if (NULL != god->refund_eh)
   {
     TMH_db->event_listen_cancel (god->refund_eh);
@@ -573,18 +580,26 @@ phase_lookup_terms (struct GetOrderData *god)
 
   /* Convert order_id to h_contract_terms */
   TMH_db->preflight (TMH_db->cls);
-  GNUNET_assert (NULL == god->contract_terms);
+  GNUNET_assert (NULL == god->contract_terms_json);
 
   {
     enum GNUNET_DB_QueryStatus qs;
 
-    qs = TMH_db->lookup_contract_terms (
+    bool paid;
+    bool wired;
+    bool session_matches;
+    qs = TMH_db->lookup_contract_terms3 (
       TMH_db->cls,
       god->hc->instance->settings.id,
       god->order_id,
-      &god->contract_terms,
+      NULL,
+      &god->contract_terms_json,
       &order_serial,
-      &db_claim_token);
+      &paid,
+      &wired,
+      &session_matches,
+      &db_claim_token,
+      &god->choice_index);
     if (0 > qs)
     {
       /* single, read-only SQL statements should never cause
@@ -607,14 +622,14 @@ phase_lookup_terms (struct GetOrderData *god)
   }
 
   /* Check if client provided the right hash code of the contract terms */
-  if (NULL != god->contract_terms)
+  if (NULL != god->contract_terms_json)
   {
     god->contract_available = true;
     if (GNUNET_YES ==
         GNUNET_is_zero (&god->h_contract_terms))
     {
       if (GNUNET_OK !=
-          TALER_JSON_contract_hash (god->contract_terms,
+          TALER_JSON_contract_hash (god->contract_terms_json,
                                     &god->h_contract_terms))
       {
         GNUNET_break (0);
@@ -630,7 +645,7 @@ phase_lookup_terms (struct GetOrderData *god)
       struct TALER_PrivateContractHashP h;
 
       if (GNUNET_OK !=
-          TALER_JSON_contract_hash (god->contract_terms,
+          TALER_JSON_contract_hash (god->contract_terms_json,
                                     &h))
       {
         GNUNET_break (0);
@@ -670,8 +685,8 @@ phase_lookup_terms (struct GetOrderData *god)
       god->order_id,
       &db_claim_token,
       &unused,
-      (NULL == god->contract_terms)
-      ? &god->contract_terms
+      (NULL == god->contract_terms_json)
+      ? &god->contract_terms_json
       : NULL);
     if (0 > qs)
     {
@@ -716,41 +731,12 @@ phase_lookup_terms (struct GetOrderData *god)
 static void
 phase_parse_contract (struct GetOrderData *god)
 {
-  struct GNUNET_JSON_Specification espec[] = {
-    TALER_JSON_spec_amount_any ("amount",
-                                &god->contract_total),
-    TALER_JSON_spec_web_url ("merchant_base_url",
-                             &god->merchant_base_url),
-    GNUNET_JSON_spec_mark_optional (
-      /* this one does NOT have to be a Web URL! */
-      GNUNET_JSON_spec_string ("fulfillment_url",
-                               &god->fulfillment_url),
-      NULL),
-    GNUNET_JSON_spec_mark_optional (
-      TALER_JSON_spec_web_url ("public_reorder_url",
-                               &god->public_reorder_url),
-      NULL),
-    GNUNET_JSON_spec_end ()
-  };
-  enum GNUNET_GenericReturnValue res;
-  const char *ename;
-  unsigned int eline;
+  god->contract_terms = TALER_MERCHANT_contract_parse (
+    god->contract_terms_json,
+    true);
 
-  GNUNET_assert (NULL != god->contract_terms);
-  if (god->contract_parsed)
-    return; /* not sure this is possible... */
-
-  res = GNUNET_JSON_parse (god->contract_terms,
-                           espec,
-                           &ename,
-                           &eline);
-  if (GNUNET_OK != res)
+  if (NULL == god->contract_terms)
   {
-    GNUNET_break (0);
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Failed to parse contract %s in DB at field %s\n",
-                god->order_id,
-                ename);
     phase_fail (god,
                 MHD_HTTP_INTERNAL_SERVER_ERROR,
                 TALER_EC_MERCHANT_GENERIC_DB_CONTRACT_CONTENT_INVALID,
@@ -758,6 +744,33 @@ phase_parse_contract (struct GetOrderData *god)
     return;
   }
   god->contract_parsed = true;
+  if ( (NULL != god->session_id) &&
+       (NULL != god->contract_terms->fulfillment_url) &&
+       (NULL == god->session_eh) )
+  {
+    struct TMH_SessionEventP session_eh = {
+      .header.size = htons (sizeof (session_eh)),
+      .header.type = htons (TALER_DBEVENT_MERCHANT_SESSION_CAPTURED),
+      .merchant_pub = god->hc->instance->merchant_pub
+    };
+
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Subscribing to session triggers for %p\n",
+                god);
+    GNUNET_CRYPTO_hash (god->session_id,
+                        strlen (god->session_id),
+                        &session_eh.h_session_id);
+    GNUNET_CRYPTO_hash (god->contract_terms->fulfillment_url,
+                        strlen (god->contract_terms->fulfillment_url),
+                        &session_eh.h_fulfillment_url);
+    god->session_eh
+      = TMH_db->event_listen (
+          TMH_db->cls,
+          &session_eh.header,
+          GNUNET_TIME_absolute_get_remaining (god->sc.long_poll_timeout),
+          &resume_by_event,
+          god);
+  }
   god->phase++;
 }
 
@@ -807,7 +820,7 @@ phase_check_client_access (struct GetOrderData *god)
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Neither claim token nor contract matched\n");
     /* Client has no rights to this order */
-    if (NULL == god->public_reorder_url)
+    if (NULL == god->contract_terms->public_reorder_url)
     {
       /* We cannot give the client a new order, just fail */
       if (! GNUNET_is_zero (&god->h_contract_terms))
@@ -851,9 +864,10 @@ phase_check_client_access (struct GetOrderData *god)
         return;
       }
       GNUNET_break (MHD_YES ==
-                    MHD_add_response_header (reply,
-                                             MHD_HTTP_HEADER_LOCATION,
-                                             god->public_reorder_url));
+                    MHD_add_response_header (
+                      reply,
+                      MHD_HTTP_HEADER_LOCATION,
+                      god->contract_terms->public_reorder_url));
       ret = MHD_queue_response (god->sc.con,
                                 MHD_HTTP_FOUND,
                                 reply);
@@ -867,8 +881,9 @@ phase_check_client_access (struct GetOrderData *god)
                TALER_MHD_REPLY_JSON_PACK (
                  god->sc.con,
                  MHD_HTTP_ACCEPTED,
-                 GNUNET_JSON_pack_string ("public_reorder_url",
-                                          god->public_reorder_url)));
+                 GNUNET_JSON_pack_string (
+                   "public_reorder_url",
+                   god->contract_terms->public_reorder_url)));
     return;
   }
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
@@ -895,7 +910,7 @@ get_order_summary (const struct GetOrderData *god)
                                                   MHD_HTTP_HEADER_ACCEPT_LANGUAGE);
   if (NULL == language_pattern)
     language_pattern = "en";
-  ret = json_string_value (TALER_JSON_extract_i18n (god->contract_terms,
+  ret = json_string_value (TALER_JSON_extract_i18n (god->contract_terms_json,
                                                     language_pattern,
                                                     "summary"));
   if (NULL == ret)
@@ -975,11 +990,11 @@ send_pay_request (struct GetOrderData *god,
     {
       struct MHD_Response *reply;
 
-      GNUNET_assert (NULL != god->fulfillment_url);
+      GNUNET_assert (NULL != god->contract_terms->fulfillment_url);
       GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                   "Redirecting to already paid order %s via fulfillment URL %s\n",
                   already_paid_order_id,
-                  god->fulfillment_url);
+                  god->contract_terms->fulfillment_url);
       reply = MHD_create_response_from_buffer (0,
                                                NULL,
                                                MHD_RESPMEM_PERSISTENT);
@@ -991,9 +1006,10 @@ send_pay_request (struct GetOrderData *god,
         return false;
       }
       GNUNET_break (MHD_YES ==
-                    MHD_add_response_header (reply,
-                                             MHD_HTTP_HEADER_LOCATION,
-                                             god->fulfillment_url));
+                    MHD_add_response_header (
+                      reply,
+                      MHD_HTTP_HEADER_LOCATION,
+                      god->contract_terms->fulfillment_url));
       {
         ret = MHD_queue_response (god->sc.con,
                                   MHD_HTTP_FOUND,
@@ -1059,7 +1075,7 @@ send_pay_request (struct GetOrderData *god,
                                taler_pay_uri),
       GNUNET_JSON_pack_allow_null (
         GNUNET_JSON_pack_string ("fulfillment_url",
-                                 god->fulfillment_url)),
+                                 god->contract_terms->fulfillment_url)),
       GNUNET_JSON_pack_allow_null (
         GNUNET_JSON_pack_string ("already_paid_order_id",
                                  already_paid_order_id)));
@@ -1116,7 +1132,7 @@ static bool
 phase_redirect_to_paid_order (struct GetOrderData *god)
 {
   if ( (NULL != god->session_id) &&
-       (NULL != god->fulfillment_url) )
+       (NULL != god->contract_terms->fulfillment_url) )
   {
     /* Check if client paid for this fulfillment article
        already within this session, but using a different
@@ -1132,11 +1148,11 @@ phase_redirect_to_paid_order (struct GetOrderData *god)
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Running re-purchase detection for %s/%s\n",
                 god->session_id,
-                god->fulfillment_url);
+                god->contract_terms->fulfillment_url);
     qs = TMH_db->lookup_order_by_fulfillment (
       TMH_db->cls,
       god->hc->instance->settings.id,
-      god->fulfillment_url,
+      god->contract_terms->fulfillment_url,
       god->session_id,
       TALER_EXCHANGE_YNA_NO != god->allow_refunded_for_repurchase,
       &already_paid_order_id);
@@ -1281,27 +1297,60 @@ static bool
 phase_check_refunded (struct GetOrderData *god)
 {
   enum GNUNET_DB_QueryStatus qs;
+  struct TALER_Amount refund_amount;
+  const char *refund_currency;
+
+  switch (god->contract_terms->version)
+  {
+  case TALER_MERCHANT_CONTRACT_VERSION_0:
+    refund_amount = god->contract_terms->details.v0.brutto;
+    refund_currency = god->contract_terms->details.v0.brutto.currency;
+    break;
+  case TALER_MERCHANT_CONTRACT_VERSION_1:
+    if (god->choice_index < 0)
+    {
+      // order was not paid, no refund to be checked
+      god->phase++;
+      return false;
+    }
+    GNUNET_assert (god->choice_index <
+                   god->contract_terms->details.v1.choices_len);
+    refund_currency = god->contract_terms->details.v1.choices[god->choice_index]
+                      .amount.currency;
+    GNUNET_assert (GNUNET_OK == TALER_amount_set_zero (refund_currency,
+                                                       &refund_amount));
+    break;
+  default:
+    {
+      GNUNET_break (0);
+      phase_fail (god,
+                  MHD_HTTP_INTERNAL_SERVER_ERROR,
+                  TALER_EC_MERCHANT_GET_ORDERS_ID_INVALID_CONTRACT_VERSION,
+                  NULL);
+      return false;
+    }
+  }
 
   if ( (god->sc.awaiting_refund) &&
        (GNUNET_OK !=
-        TALER_amount_cmp_currency (&god->contract_total,
+        TALER_amount_cmp_currency (&refund_amount,
                                    &god->sc.refund_expected)) )
   {
     GNUNET_break (0);
     phase_fail (god,
                 MHD_HTTP_CONFLICT,
                 TALER_EC_MERCHANT_GENERIC_CURRENCY_MISMATCH,
-                god->contract_total.currency);
+                refund_currency);
     return false;
   }
 
   /* At this point, we know the contract was paid. Let's check for
      refunds. First, clear away refunds found from previous invocations. */
   GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_set_zero (god->contract_total.currency,
+                 TALER_amount_set_zero (refund_currency,
                                         &god->refund_amount));
   GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_set_zero (god->contract_total.currency,
+                 TALER_amount_set_zero (refund_currency,
                                         &god->refund_taken));
   qs = TMH_db->lookup_refunds_detailed (
     TMH_db->cls,
@@ -1430,7 +1479,8 @@ phase_return_status (struct GetOrderData *god)
                  MHD_HTTP_OK,
                  GNUNET_JSON_pack_allow_null (
                    GNUNET_JSON_pack_string ("fulfillment_url",
-                                            god->fulfillment_url)),
+                                            god->contract_terms->fulfillment_url
+                                            )),
                  GNUNET_JSON_pack_bool ("refunded",
                                         god->refunded),
                  GNUNET_JSON_pack_bool ("refund_pending",
@@ -1447,8 +1497,8 @@ phase_return_status (struct GetOrderData *god)
     char *qr;
     char *uri;
 
-    GNUNET_assert (NULL != god->contract_terms);
-    uri = make_taler_refund_uri (god->merchant_base_url,
+    GNUNET_assert (NULL != god->contract_terms_json);
+    uri = make_taler_refund_uri (god->contract_terms->merchant_base_url,
                                  god->order_id);
     if (NULL == uri)
     {
@@ -1511,7 +1561,7 @@ phase_return_status (struct GetOrderData *god)
 
     context = GNUNET_JSON_PACK (
       GNUNET_JSON_pack_object_incref ("contract_terms",
-                                      god->contract_terms),
+                                      god->contract_terms_json),
       GNUNET_JSON_pack_string ("order_summary",
                                get_order_summary (god)),
       TALER_JSON_pack_amount ("refund_amount",
