@@ -19,7 +19,7 @@
  * @author Priscilla HUANG
  */
 #include "platform.h"
-#include "taler-merchant-httpd_private-get-templates-ID.h"
+#include "taler-merchant-httpd_get-templates-ID.h"
 #include <taler/taler_json_lib.h>
 
 
