@@ -56,44 +56,22 @@ TMH_MHD_handler_agpl_redirect (const struct TMH_RequestHandler *rh,
 bool
 TMH_MHD_test_html_desired (struct MHD_Connection *connection)
 {
-  bool ret = false;
   const char *accept;
+  double json_q;
+  double html_q;
 
-  // FIXME: use TALER_MHD_check_accept here!
   accept = MHD_lookup_connection_value (connection,
                                         MHD_HEADER_KIND,
                                         MHD_HTTP_HEADER_ACCEPT);
-  if (NULL != accept)
-  {
-    char *a = GNUNET_strdup (accept);
-    char *saveptr;
-
-    for (char *t = strtok_r (a, ",", &saveptr);
-         NULL != t;
-         t = strtok_r (NULL, ",", &saveptr))
-    {
-      char *end;
-
-      /* skip leading whitespace */
-      while (isspace ((unsigned char) t[0]))
-        t++;
-      /* trim of ';q=' parameter and everything after space */
-      end = strchr (t, ';');
-      if (NULL != end)
-        *end = '\0';
-      end = strchr (t, ' ');
-      if (NULL != end)
-        *end = '\0';
-      if (0 == strcasecmp ("text/html",
-                           t))
-      {
-        ret = true;
-        break;
-      }
-    }
-    GNUNET_free (a);
-  }
-  return ret;
+  if (NULL == accept)
+    return false; /* nothing selected, we read this as not HTML */
+  html_q = TALER_pattern_matches (accept,
+                                  "text/html");
+  json_q = TALER_pattern_matches (accept,
+                                  "application/json");
+  if (html_q > json_q)
+    return true;
+  return false;
 }
 
 
