@@ -28,6 +28,7 @@
 #include <taler/taler_exchange_service.h>
 #include "taler-merchant-httpd_exchanges.h"
 #include "taler-merchant-httpd_helper.h"
+#include "taler-merchant-httpd_post-orders-ID-abort.h"
 
 
 /**
@@ -114,6 +115,10 @@ struct RefundDetails
    */
   bool processed;
 
+  /**
+   * Did we find the deposit in our own database?
+   */
+  bool found_deposit;
 };
 
 
@@ -361,13 +366,25 @@ generate_success_response (struct AbortContext *ac)
     struct RefundDetails *rdi = &ac->rd[i];
     json_t *detail;
 
-    if ( ( (MHD_HTTP_BAD_REQUEST <= rdi->http_status) &&
-           (MHD_HTTP_NOT_FOUND != rdi->http_status) &&
-           (MHD_HTTP_GONE != rdi->http_status) ) ||
-         (0 == rdi->http_status) ||
-         (NULL == rdi->exchange_reply) )
-      hc = MHD_HTTP_BAD_GATEWAY;
-    if (MHD_HTTP_OK != rdi->http_status)
+    if (rdi->found_deposit)
+    {
+      if ( ( (MHD_HTTP_BAD_REQUEST <= rdi->http_status) &&
+             (MHD_HTTP_NOT_FOUND != rdi->http_status) &&
+             (MHD_HTTP_GONE != rdi->http_status) ) ||
+           (0 == rdi->http_status) ||
+           (NULL == rdi->exchange_reply) )
+      {
+        hc = MHD_HTTP_BAD_GATEWAY;
+      }
+    }
+    if (! rdi->found_deposit)
+    {
+      detail = GNUNET_JSON_PACK (
+        GNUNET_JSON_pack_string ("type",
+                                 "undeposited"));
+    }
+    else if (MHD_HTTP_OK != rdi->http_status)
+    {
       detail = GNUNET_JSON_PACK (
         GNUNET_JSON_pack_string ("type",
                                  "failure"),
@@ -383,7 +400,9 @@ generate_success_response (struct AbortContext *ac)
         GNUNET_JSON_pack_allow_null (
           GNUNET_JSON_pack_object_incref ("exchange_reply",
                                           rdi->exchange_reply)));
-    else
+    }
+    else if (rdi->found_deposit)
+    {
       detail = GNUNET_JSON_PACK (
         GNUNET_JSON_pack_string ("type",
                                  "success"),
@@ -393,6 +412,7 @@ generate_success_response (struct AbortContext *ac)
                                     &rdi->exchange_sig),
         GNUNET_JSON_pack_data_auto ("exchange_pub",
                                     &rdi->exchange_pub));
+    }
     GNUNET_assert (0 ==
                    json_array_append_new (refunds,
                                           detail));
@@ -534,6 +554,11 @@ process_abort_with_exchange (void *cls,
       continue;
     rdi->processed = true;
     ac->pending--;
+    if (! rdi->found_deposit)
+    {
+      /* Coin wasn't even deposited yet, we do not need to refund it. */
+      continue;
+    }
     rdi->rh = TALER_EXCHANGE_refund (
       TMH_curl_ctx,
       ac->current_exchange,
@@ -556,6 +581,9 @@ process_abort_with_exchange (void *cls,
     }
     ac->pending_at_ce++;
   }
+  /* Still continue if no coins for this exchange were deposited. */
+  if (0 == ac->pending_at_ce)
+    find_next_exchange (ac);
 }
 
 
@@ -592,11 +620,12 @@ find_next_exchange (struct AbortContext *ac)
                                             ac);
       if (NULL == ac->fo)
       {
+        /* strange, should have happened on pay! */
         GNUNET_break (0);
         resume_abort_with_error (ac,
                                  MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                 TALER_EC_MERCHANT_POST_ORDERS_ID_ABORT_EXCHANGE_LOOKUP_FAILED,
-                                 "Failed to lookup exchange by URL");
+                                 TALER_EC_MERCHANT_GENERIC_EXCHANGE_UNTRUSTED,
+                                 ac->current_exchange);
         return;
       }
       return;
@@ -631,7 +660,6 @@ refund_coins (void *cls,
   struct AbortContext *ac = cls;
   struct GNUNET_TIME_Timestamp now;
 
-  (void) amount_with_fee;
   (void) deposit_fee;
   (void) refund_fee;
   now = GNUNET_TIME_timestamp_get ();
@@ -647,7 +675,8 @@ refund_coins (void *cls,
           strcmp (exchange_url,
                   rdi->exchange_url)) )
       continue; /* not in request */
-
+    rdi->found_deposit = true;
+    rdi->amount_with_fee = *amount_with_fee;
     /* Store refund in DB */
     qs = TMH_db->refund_coin (TMH_db->cls,
                               ac->hc->instance->settings.id,
@@ -882,9 +911,6 @@ parse_abort (struct MHD_Connection *connection,
       struct RefundDetails *rd = &ac->rd[coins_index];
       const char *exchange_url;
       struct GNUNET_JSON_Specification ispec[] = {
-        TALER_JSON_spec_amount ("contribution",
-                                TMH_currency,
-                                &rd->amount_with_fee),
         TALER_JSON_spec_web_url ("exchange_url",
                                  &exchange_url),
         GNUNET_JSON_spec_fixed_auto ("coin_pub",
