@@ -619,7 +619,6 @@ phase_fetch_contract (struct GetOrderRequestContext *gorc)
     gorc->order_only = false;
   }
   TMH_db->preflight (TMH_db->cls);
-  /* TODO: Check if choice_index is actually set to NULL if not in db. */
   qs = TMH_db->lookup_contract_terms3 (TMH_db->cls,
                                        hc->instance->settings.id,
                                        hc->infix,
@@ -1387,7 +1386,7 @@ phase_reply_result (struct GetOrderRequestContext *gorc)
   struct TMH_HandlerContext *hc = gorc->hc;
   MHD_RESULT ret;
   char *order_status_url;
-  json_t *choice_index = json_null();
+  json_t *choice_index;
 
   {
     struct TALER_PrivateContractHashP *h_contract = NULL;
@@ -1415,12 +1414,17 @@ phase_reply_result (struct GetOrderRequestContext *gorc)
                   TALER_amount_is_zero (&gorc->contract_amount));
     gorc->last_payment = gorc->timestamp;
   }
-  if (-1 != gorc->choice_index) {
+  if (-1 != gorc->choice_index)
+  {
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                 "Choice index is %d\n",
                 gorc->choice_index);
     choice_index = json_integer ((json_int_t) gorc->choice_index);
   }
+  else
+  {
+    choice_index = NULL;
+  }
   ret = TALER_MHD_REPLY_JSON_PACK (
     gorc->sc.con,
     MHD_HTTP_OK,
@@ -1459,10 +1463,10 @@ phase_reply_result (struct GetOrderRequestContext *gorc)
                                   gorc->refund_details),
     GNUNET_JSON_pack_string ("order_status_url",
                              order_status_url),
-    {
-      .field_name = "choice_index",
-      .object = choice_index,
-    });
+    GNUNET_JSON_pack_allow_null (
+      GNUNET_JSON_pack_object_steal (
+        "choice_index",
+        choice_index)));
   GNUNET_free (order_status_url);
   gorc->wire_details = NULL;
   gorc->refund_details = NULL;
