@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2014-2021 Taler Systems SA
+  (C) 2014-2024 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -46,7 +46,7 @@ static void
 transfer_cb (void *cls,
              const struct TALER_Amount *credit_amount,
              const struct TALER_WireTransferIdentifierRawP *wtid,
-             const char *payto_uri,
+             struct TALER_FullPayto payto_uri,
              const char *exchange_url,
              uint64_t transfer_serial_id,
              struct GNUNET_TIME_Timestamp execution_time,
@@ -61,8 +61,8 @@ transfer_cb (void *cls,
                             credit_amount),
     GNUNET_JSON_pack_data_auto ("wtid",
                                 wtid),
-    GNUNET_JSON_pack_string ("payto_uri",
-                             payto_uri),
+    TALER_JSON_pack_full_payto ("payto_uri",
+                                payto_uri),
     GNUNET_JSON_pack_string ("exchange_url",
                              exchange_url),
     GNUNET_JSON_pack_uint64 ("transfer_serial_id",
@@ -96,7 +96,9 @@ TMH_private_get_transfers (const struct TMH_RequestHandler *rh,
                            struct MHD_Connection *connection,
                            struct TMH_HandlerContext *hc)
 {
-  const char *payto_uri;
+  struct TALER_FullPayto payto_uri = {
+    .full_payto = NULL
+  };
   struct GNUNET_TIME_Timestamp before = GNUNET_TIME_UNIT_FOREVER_TS;
   struct GNUNET_TIME_Timestamp after = GNUNET_TIME_UNIT_ZERO_TS;
   int64_t limit = -20;
@@ -104,9 +106,19 @@ TMH_private_get_transfers (const struct TMH_RequestHandler *rh,
   enum TALER_EXCHANGE_YesNoAll verified;
 
   (void) rh;
-  payto_uri = MHD_lookup_connection_value (connection,
-                                           MHD_GET_ARGUMENT_KIND,
-                                           "payto_uri");
+  {
+    const char *esc_payto;
+
+    esc_payto = MHD_lookup_connection_value (connection,
+                                             MHD_GET_ARGUMENT_KIND,
+                                             "payto_uri");
+    if (NULL != esc_payto)
+    {
+      payto_uri.full_payto
+        = GNUNET_strdup (esc_payto);
+      (void) MHD_http_unescape (payto_uri.full_payto);
+    }
+  }
   {
     const char *before_s;
 
@@ -173,6 +185,7 @@ TMH_private_get_transfers (const struct TMH_RequestHandler *rh,
                                    verified,
                                    &transfer_cb,
                                    ja);
+    GNUNET_free (payto_uri.full_payto);
     if (0 > qs)
     {
       /* Simple select queries should not cause serialization issues */
