@@ -177,7 +177,7 @@ TMH_private_post_products (const struct TMH_RequestHandler *rh,
   }
 
   if (NULL == pd.image)
-    pd.image = "";
+    pd.image = (char *) "";
   if (! TMH_image_data_url_valid (pd.image))
   {
     GNUNET_break_op (0);
