@@ -22,6 +22,7 @@
 #include <gnunet/gnunet_util_lib.h>
 #include <jansson.h>
 #include <pthread.h>
+#include "taler_merchant_util.h"
 #include "taler_merchantdb_lib.h"
 #include "taler_merchantdb_plugin.h"
 #include <taler/taler_dbevents.h>
@@ -42,7 +43,7 @@ static struct WorkResponse *w_head;
 
 static struct WorkResponse *w_tail;
 
-static struct GNUNET_DB_EventHandler *eh;
+static struct GNUNET_DB_EventHandler *event_handler;
 
 /**
  * The merchant's configuration.
@@ -93,10 +94,10 @@ shutdown_task (void *cls)
   (void) cls;
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Running shutdown\n");
-  if (NULL != eh)
+  if (NULL != event_handler)
   {
-    db_plugin->event_listen_cancel (eh);
-    eh = NULL;
+    db_plugin->event_listen_cancel (event_handler);
+    event_handler = NULL;
   }
   if (NULL != task)
   {
@@ -279,11 +280,11 @@ pending_webhooks_cb (void *cls,
 {
   struct WorkResponse *w = GNUNET_new (struct WorkResponse);
   CURL *eh;
+  struct curl_slist *job_headers = NULL;
+
   (void) retries;
   (void) next_attempt;
   (void) cls;
-  struct curl_slist *job_headers = NULL;
-
   GNUNET_CONTAINER_DLL_insert (w_head,
                                w_tail,
                                w);
@@ -522,7 +523,7 @@ run (void *cls,
       db_plugin->connect (db_plugin->cls))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Failed to connect to database\n");
+                "Failed to connect to database. Consider running taler-merchant-dbinit!\n");
     GNUNET_SCHEDULER_shutdown ();
     global_ret = EXIT_FAILURE;
     return;
@@ -533,11 +534,11 @@ run (void *cls,
       .type = htons (TALER_DBEVENT_MERCHANT_WEBHOOK_PENDING)
     };
 
-    eh = db_plugin->event_listen (db_plugin->cls,
-                                  &es,
-                                  GNUNET_TIME_UNIT_FOREVER_REL,
-                                  &db_notify,
-                                  NULL);
+    event_handler = db_plugin->event_listen (db_plugin->cls,
+                                             &es,
+                                             GNUNET_TIME_UNIT_FOREVER_REL,
+                                             &db_notify,
+                                             NULL);
   }
   GNUNET_assert (NULL == task);
   task = GNUNET_SCHEDULER_add_now (&select_work,
@@ -567,19 +568,14 @@ main (int argc,
   };
   enum GNUNET_GenericReturnValue ret;
 
-  if (GNUNET_OK !=
-      GNUNET_STRINGS_get_utf8_args (argc, argv,
-                                    &argc, &argv))
-    return EXIT_INVALIDARGUMENT;
-  TALER_OS_init ();
   ret = GNUNET_PROGRAM_run (
+    TALER_MERCHANT_project_data (),
     argc, argv,
     "taler-merchant-webhook",
     gettext_noop (
       "background process that executes webhooks"),
     options,
     &run, NULL);
-  GNUNET_free_nz ((void *) argv);
   if (GNUNET_SYSERR == ret)
     return EXIT_INVALIDARGUMENT;
   if (GNUNET_NO == ret)
