@@ -23,6 +23,7 @@
 #include <jansson.h>
 #include <pthread.h>
 #include <taler/taler_dbevents.h>
+#include "taler_merchant_util.h"
 #include "taler_merchant_bank_lib.h"
 #include "taler_merchantdb_lib.h"
 #include "taler_merchantdb_plugin.h"
@@ -68,7 +69,7 @@ struct Watch
   /**
    * For which account are we importing bank transfers?
    */
-  char *payto_uri;
+  struct TALER_FullPayto payto_uri;
 
   /**
    * Bank history request.
@@ -189,7 +190,7 @@ save (struct Watch *w)
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                 "Failed to persist wirewatch progress for %s/%s (%d)\n",
                 w->instance_id,
-                w->payto_uri,
+                w->payto_uri.full_payto,
                 qs);
     GNUNET_SCHEDULER_shutdown ();
     global_ret = EXIT_FAILURE;
@@ -216,7 +217,7 @@ end_watch (struct Watch *w)
     w->hh = NULL;
   }
   GNUNET_free (w->instance_id);
-  GNUNET_free (w->payto_uri);
+  GNUNET_free (w->payto_uri.full_payto);
   TALER_MERCHANT_BANK_auth_free (&w->ad);
   GNUNET_CONTAINER_DLL_remove (w_head,
                                w_tail,
@@ -351,7 +352,6 @@ credit_cb (
       enum GNUNET_DB_QueryStatus qs;
       char *exchange_url;
       struct TALER_WireTransferIdentifierRawP wtid;
-      char *credit_payto;
 
       GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                   "Received wire transfer `%s' over %s\n",
@@ -372,13 +372,12 @@ credit_cb (
       }
       /* FIXME-Performance-Optimization: consider grouping multiple inserts
          into one bigger transaction with just one notify. */
-      credit_payto = TALER_payto_normalize (details->credit_account_uri);
       qs = db_plugin->insert_transfer (db_plugin->cls,
                                        w->instance_id,
                                        exchange_url,
                                        &wtid,
                                        &details->amount,
-                                       credit_payto,
+                                       details->credit_account_uri,
                                        true /* confirmed */);
       if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
       {
@@ -404,7 +403,7 @@ credit_cb (
           GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                       "Inserting transfer for %s into database failed. Is the credit account %s configured correctly?\n",
                       w->instance_id,
-                      credit_payto);
+                      details->credit_account_uri.full_payto);
         }
         if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT == qs)
         {
@@ -425,7 +424,6 @@ credit_cb (
           }
         }
       }
-      GNUNET_free (credit_payto);
       GNUNET_free (exchange_url);
       if (qs < 0)
       {
@@ -553,7 +551,7 @@ static void
 start_watch (
   void *cls,
   const char *instance,
-  const char *payto_uri,
+  struct TALER_FullPayto payto_uri,
   const char *credit_facade_url,
   const json_t *credit_facade_credentials,
   uint64_t last_serial)
@@ -570,7 +568,7 @@ start_watch (
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Failed to parse authentication data of `%s/%s'\n",
                 instance,
-                payto_uri);
+                payto_uri.full_payto);
     GNUNET_free (w);
     GNUNET_SCHEDULER_shutdown ();
     global_ret = EXIT_NOTCONFIGURED;
@@ -581,7 +579,7 @@ start_watch (
                                w_tail,
                                w);
   w->instance_id = GNUNET_strdup (instance);
-  w->payto_uri = TALER_payto_normalize (payto_uri);
+  w->payto_uri.full_payto = GNUNET_strdup (payto_uri.full_payto);
   w->start_row = last_serial;
   w->task = GNUNET_SCHEDULER_add_now (&do_work,
                                       w);
@@ -657,7 +655,7 @@ run (void *cls,
       db_plugin->connect (db_plugin->cls))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Failed to connect to database\n");
+                "Failed to connect to database. Consider running taler-merchant-dbinit!\n");
     GNUNET_SCHEDULER_shutdown ();
     global_ret = EXIT_FAILURE;
     return;
@@ -728,14 +726,10 @@ main (int argc,
   };
   enum GNUNET_GenericReturnValue ret;
 
-  if (GNUNET_OK !=
-      GNUNET_STRINGS_get_utf8_args (argc, argv,
-                                    &argc, &argv))
-    return EXIT_INVALIDARGUMENT;
-  TALER_OS_init ();
   do {
     config_changed_flag = false;
     ret = GNUNET_PROGRAM_run (
+      TALER_MERCHANT_project_data (),
       argc, argv,
       "taler-merchant-wirewatch",
       gettext_noop (
@@ -744,7 +738,6 @@ main (int argc,
       &run, NULL);
   } while ( (1 == persist_mode) &&
             config_changed_flag);
-  GNUNET_free_nz ((void *) argv);
   if (GNUNET_SYSERR == ret)
     return EXIT_INVALIDARGUMENT;
   if (GNUNET_NO == ret)
