@@ -1,6 +1,6 @@
 /*
    This file is part of TALER
-   Copyright (C) 2022 Taler Systems SA
+   Copyright (C) 2022-2024 Taler Systems SA
 
    TALER is free software; you can redistribute it and/or modify it under the
    terms of the GNU General Public License as published by the Free Software
@@ -17,6 +17,7 @@
  * @file backenddb/pg_account_kyc_set_status.c
  * @brief Implementation of the account_kyc_set_status function for Postgres
  * @author Iván Ávalos
+ * @author Christian Grothoff
  */
 #include "platform.h"
 #include <taler/taler_error_codes.h>
@@ -32,44 +33,47 @@ TMH_PG_account_kyc_set_status (
   const char *merchant_id,
   const struct TALER_MerchantWireHashP *h_wire,
   const char *exchange_url,
-  uint64_t exchange_kyc_serial,
   struct GNUNET_TIME_Timestamp timestamp,
   unsigned int exchange_http_status,
   enum TALER_ErrorCode exchange_ec_code,
   const struct TALER_AccountAccessTokenP *access_token,
-  unsigned int num_limits,
-  const struct TALER_MERCHANTDB_DepositLimits *limits,
+  const json_t *jlimits,
   bool in_aml_review,
   bool kyc_ok)
 {
   struct PostgresClosure *pg = cls;
+  struct TALER_MERCHANTDB_MerchantKycStatusChangeEventP ev = {
+    .header.size = htons (sizeof (ev)),
+    .header.type = htons (TALER_DBEVENT_MERCHANT_EXCHANGE_KYC_STATUS_CHANGED),
+    .h_wire = *h_wire
+  };
+  struct GNUNET_DB_EventHeaderP hdr = {
+    .size = htons (sizeof (hdr)),
+    .type = htons (TALER_DBEVENT_MERCHANT_KYC_STATUS_CHANGED)
+  };
+  char *notify_s
+    = GNUNET_PQ_get_event_notify_channel (&ev.header);
+  char *notify2_s
+    = GNUNET_PQ_get_event_notify_channel (&hdr);
   uint32_t http_status32 = (uint32_t) exchange_http_status;
   uint32_t ec_code32 = (uint32_t) exchange_ec_code;
-  struct TALER_Amount thresholds[GNUNET_NZL (num_limits)];
-  struct GNUNET_TIME_Relative timeframes[GNUNET_NZL (num_limits)];
-  bool soft_limits[GNUNET_NZL (num_limits)];
   struct GNUNET_PQ_QueryParam params[] = {
     GNUNET_PQ_query_param_string (merchant_id),
     GNUNET_PQ_query_param_auto_from_type (h_wire),
     GNUNET_PQ_query_param_string (exchange_url),
     GNUNET_PQ_query_param_timestamp (&timestamp),
-    GNUNET_PQ_query_param_uint64 (&exchange_kyc_serial),
     GNUNET_PQ_query_param_uint32 (&http_status32),
     GNUNET_PQ_query_param_uint32 (&ec_code32),
     NULL != access_token
     ? GNUNET_PQ_query_param_auto_from_type (access_token)
     : GNUNET_PQ_query_param_null (),
-    TALER_PQ_query_param_array_amount (num_limits,
-                                       thresholds,
-                                       pg->conn),
-    GNUNET_PQ_query_param_array_rel_time (num_limits,
-                                          timeframes,
-                                          pg->conn),
-    GNUNET_PQ_query_param_array_bool (num_limits,
-                                      soft_limits,
-                                      pg->conn),
+    NULL != jlimits
+    ? TALER_PQ_query_param_json (jlimits)
+    : GNUNET_PQ_query_param_null (),
     GNUNET_PQ_query_param_bool (in_aml_review),
     GNUNET_PQ_query_param_bool (kyc_ok),
+    GNUNET_PQ_query_param_string (notify_s),
+    GNUNET_PQ_query_param_string (notify2_s),
     GNUNET_PQ_query_param_end
   };
   bool no_instance;
@@ -90,24 +94,21 @@ TMH_PG_account_kyc_set_status (
            "  out_no_instance AS no_instance"
            " ,out_no_account AS no_account"
            " FROM merchant_do_account_kyc_set_status"
-           "($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13);");
-  for (unsigned int i=0; i<num_limits; i++)
-  {
-    const struct TALER_MERCHANTDB_DepositLimits *limit
-      = &limits[i];
-
-    thresholds[i] = limit->threshold;
-    timeframes[i] = limit->timeframe;
-    soft_limits[i] = limit->soft_limit;
-  }
+           "($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12);");
   qs = GNUNET_PQ_eval_prepared_singleton_select (
     pg->conn,
     "account_kyc_set_status",
     params,
     rs);
-  GNUNET_PQ_cleanup_query_params_closures (params);
+  GNUNET_free (notify_s);
+  GNUNET_free (notify2_s);
   if (qs <= 0)
+  {
+    GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR != qs);
+    GNUNET_break (GNUNET_DB_STATUS_HARD_ERROR != qs);
+    GNUNET_break (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS != qs);
     return qs;
+  }
   GNUNET_break (! no_instance);
   GNUNET_break (! no_account);
   return qs;
