@@ -32,13 +32,11 @@
  * @param merchant_id merchant backend instance ID
  * @param h_wire hash of the wire account to check
  * @param exchange_url base URL of the exchange to check
- * @param exchange_kyc_serial serial number for our account at the exchange (0 if unknown)
  * @param timestamp timestamp to store
  * @param exchange_http_status HTTP status code returned last by the exchange
  * @param exchange_ec_code Taler error code returned last by the exchange
  * @param access_token access token for the KYC process, NULL for none
- * @param num_limits length of the @a limits array
- * @param limits array with deposit limits returned by the exchange
+ * @param jlimits JSON array with AccountLimits returned by the exchange
  * @param in_aml_review true if the exchange says the account is under review
  * @param kyc_ok current KYC status (true for satisfied)
  * @return database result code
@@ -49,13 +47,11 @@ TMH_PG_account_kyc_set_status (
   const char *merchant_id,
   const struct TALER_MerchantWireHashP *h_wire,
   const char *exchange_url,
-  uint64_t exchange_kyc_serial,
   struct GNUNET_TIME_Timestamp timestamp,
   unsigned int exchange_http_status,
   enum TALER_ErrorCode exchange_ec_code,
   const struct TALER_AccountAccessTokenP *access_token,
-  unsigned int num_limits,
-  const struct TALER_MERCHANTDB_DepositLimits *limits,
+  const json_t *jlimits,
   bool in_aml_review,
   bool kyc_ok);
 
