@@ -35,6 +35,10 @@ DECLARE
   my_decose INT8;
 DECLARE
   my_order_serial INT8;
+DECLARE
+  my_merchant_serial INT8;
+DECLARE
+  my_order_id TEXT;
 BEGIN
 
 -- Find exchange sign key
@@ -157,4 +161,20 @@ UPDATE merchant_contract_terms
       WHERE mdc.wire_pending
         AND mdc.order_serial=my_order_serial);
 
+-- POSSIBLE LOCATION FOR THE WIRE WEBHOOK OF ORDER
+--
+-- INSERT INTO merchant_pending_webhooks
+-- (merchant_serial
+-- ,webhook_serial
+-- ,url
+-- ,http_method
+-- ,body)
+-- SELECT mw.merchant_serial
+--      ,mw.webhook_serial
+--      ,mw.url
+--      ,mw.http_method
+--      ,json_build_object('order_id', my_order_id)::TEXT
+-- FROM merchant_webhook mw
+-- WHERE mw.event_type = 'order_settled'
+--   AND mw.merchant_serial = my_merchant_serial;
 END $$;
