@@ -45,9 +45,6 @@ TMH_PG_insert_template (void *cls,
     (NULL == td->editable_defaults)
     ? GNUNET_PQ_query_param_null ()
     : TALER_PQ_query_param_json (td->editable_defaults),
-    (NULL == td->required_currency)
-    ? GNUNET_PQ_query_param_null ()
-    : GNUNET_PQ_query_param_string (td->required_currency),
     GNUNET_PQ_query_param_end
   };
 
@@ -61,10 +58,9 @@ TMH_PG_insert_template (void *cls,
            ",otp_device_id"
            ",template_contract"
            ",editable_defaults"
-           ",required_currency"
            ")"
            " SELECT merchant_serial,"
-           " $2, $3, $4, $5, $6, $7"
+           " $2, $3, $4, $5, $6"
            " FROM merchant_instances"
            " WHERE merchant_id=$1");
   return GNUNET_PQ_eval_prepared_non_select (pg->conn,
