@@ -33,7 +33,7 @@ TMH_PG_insert_transfer (
   const char *exchange_url,
   const struct TALER_WireTransferIdentifierRawP *wtid,
   const struct TALER_Amount *credit_amount,
-  const char *payto_uri,
+  struct TALER_FullPayto payto_uri,
   bool confirmed)
 {
   struct PostgresClosure *pg = cls;
@@ -42,7 +42,7 @@ TMH_PG_insert_transfer (
     GNUNET_PQ_query_param_auto_from_type (wtid),
     TALER_PQ_query_param_amount_with_currency (pg->conn,
                                                credit_amount),
-    GNUNET_PQ_query_param_string (payto_uri),
+    GNUNET_PQ_query_param_string (payto_uri.full_payto),
     GNUNET_PQ_query_param_bool (confirmed),
     GNUNET_PQ_query_param_string (instance_id),
     GNUNET_PQ_query_param_end
