@@ -37,7 +37,7 @@ TMH_PG_insert_transfer_details (
   void *cls,
   const char *instance_id,
   const char *exchange_url,
-  const char *payto_uri,
+  struct TALER_FullPayto payto_uri,
   const struct TALER_WireTransferIdentifierRawP *wtid,
   const struct TALER_EXCHANGE_TransferData *td)
 {
@@ -87,7 +87,7 @@ TMH_PG_insert_transfer_details (
       struct GNUNET_PQ_QueryParam params[] = {
         GNUNET_PQ_query_param_string (instance_id),
         GNUNET_PQ_query_param_string (exchange_url),
-        GNUNET_PQ_query_param_string (payto_uri),
+        GNUNET_PQ_query_param_string (payto_uri.full_payto),
         GNUNET_PQ_query_param_auto_from_type (wtid),
         GNUNET_PQ_query_param_timestamp (&td->execution_time),
         GNUNET_PQ_query_param_auto_from_type (&td->exchange_pub),
