@@ -1,6 +1,6 @@
 /*
    This file is part of TALER
-   Copyright (C) 2023 Taler Systems SA
+   Copyright (C) 2023, 2024 Taler Systems SA
 
    TALER is free software; you can redistribute it and/or modify it under the
    terms of the GNU General Public License as published by the Free Software
@@ -25,11 +25,13 @@
 #include "pg_lookup_token_family.h"
 #include "pg_helper.h"
 
+
 enum GNUNET_DB_QueryStatus
-TMH_PG_lookup_token_family (void *cls,
-                            const char *instance_id,
-                            const char *token_family_slug,
-                            struct TALER_MERCHANTDB_TokenFamilyDetails *details)
+TMH_PG_lookup_token_family (
+  void *cls,
+  const char *instance_id,
+  const char *token_family_slug,
+  struct TALER_MERCHANTDB_TokenFamilyDetails *details)
 {
   struct PostgresClosure *pg = cls;
   struct GNUNET_PQ_QueryParam params[] = {
@@ -62,12 +64,20 @@ TMH_PG_lookup_token_family (void *cls,
                                     &details->description),
       TALER_PQ_result_spec_json ("description_i18n",
                                  &details->description_i18n),
+      GNUNET_PQ_result_spec_allow_null (
+        TALER_PQ_result_spec_json ("extra_data",
+                                   &details->extra_data),
+        NULL),
       GNUNET_PQ_result_spec_timestamp ("valid_after",
                                        &details->valid_after),
       GNUNET_PQ_result_spec_timestamp ("valid_before",
                                        &details->valid_before),
       GNUNET_PQ_result_spec_relative_time ("duration",
                                            &details->duration),
+      GNUNET_PQ_result_spec_relative_time ("validity_granularity",
+                                           &details->validity_granularity),
+      GNUNET_PQ_result_spec_relative_time ("start_offset",
+                                           &details->start_offset),
       GNUNET_PQ_result_spec_string ("kind",
                                     &kind),
       GNUNET_PQ_result_spec_uint64 ("issued",
@@ -86,9 +96,12 @@ TMH_PG_lookup_token_family (void *cls,
              ",name"
              ",description"
              ",description_i18n"
+             ",extra_data"
              ",valid_after"
              ",valid_before"
              ",duration"
+             ",validity_granularity"
+             ",start_offset"
              ",kind"
              ",issued"
              ",used"
@@ -97,6 +110,9 @@ TMH_PG_lookup_token_family (void *cls,
              "   USING (merchant_serial)"
              " WHERE merchant_instances.merchant_id=$1"
              "   AND merchant_token_families.slug=$2");
+    memset (details,
+            0,
+            sizeof (*details));
     qs = GNUNET_PQ_eval_prepared_singleton_select (pg->conn,
                                                    "lookup_token_family",
                                                    params,
