@@ -34,16 +34,16 @@ TMH_PG_lookup_token_family_key (
   void *cls,
   const char *instance_id,
   const char *token_family_slug,
-  struct GNUNET_TIME_Timestamp min_valid_after,
-  struct GNUNET_TIME_Timestamp max_valid_after,
+  struct GNUNET_TIME_Timestamp valid_at,
+  struct GNUNET_TIME_Timestamp sign_until,
   struct TALER_MERCHANTDB_TokenFamilyKeyDetails *details)
 {
   struct PostgresClosure *pg = cls;
   struct GNUNET_PQ_QueryParam params[] = {
     GNUNET_PQ_query_param_string (instance_id),
     GNUNET_PQ_query_param_string (token_family_slug),
-    GNUNET_PQ_query_param_timestamp (&min_valid_after),
-    GNUNET_PQ_query_param_timestamp (&max_valid_after),
+    GNUNET_PQ_query_param_timestamp (&valid_at),
+    GNUNET_PQ_query_param_timestamp (&sign_until),
     GNUNET_PQ_query_param_end
   };
 
@@ -54,29 +54,33 @@ TMH_PG_lookup_token_family_key (
            " h_pub"
            ",pub"
            ",priv"
-           ",cipher"
-           ",merchant_token_family_keys.valid_after as key_valid_after"
-           ",merchant_token_family_keys.valid_before as key_valid_before"
+           ",cipher_choice"
+           ",mtfk.signature_validity_start"
+           ",mtfk.signature_validity_end"
+           ",mtfk.private_key_deleted_at"
            ",slug"
            ",name"
            ",description"
            ",description_i18n"
-           ",merchant_token_families.valid_after"
-           ",merchant_token_families.valid_before"
+           ",mtf.valid_after"
+           ",mtf.valid_before"
            ",duration"
+           ",validity_granularity"
+           ",start_offset"
            ",kind"
            ",issued"
            ",used"
-           " FROM merchant_token_families"
-           " LEFT JOIN merchant_token_family_keys"
-           "   ON merchant_token_families.token_family_serial = merchant_token_family_keys.token_family_serial"
-           "   AND merchant_token_family_keys.valid_after >= $3"
-           "   AND merchant_token_family_keys.valid_after <= $4"  /* TODO: Should this be < instead of <=? */
-           " JOIN merchant_instances"
+           " FROM merchant_token_families mtf"
+           " LEFT JOIN merchant_token_family_keys mtfk"
+           "  USING (token_family_serial)"
+           " JOIN merchant_instances mi"
            "   USING (merchant_serial)"
-           " WHERE merchant_instances.merchant_id=$1"
+           " WHERE mi.merchant_id=$1"
            "   AND slug=$2"
-           " ORDER BY merchant_token_family_keys.valid_after ASC"
+           "   AND COALESCE ($3 >= mtfk.signature_validity_start, TRUE)"
+           "   AND COALESCE ($3 <= mtfk.signature_validity_end, TRUE)"
+           "   AND COALESCE ($4 <= mtfk.private_key_deleted_at, TRUE)"
+           " ORDER BY mtfk.signature_validity_start ASC"
            " LIMIT 1");
 
   if (NULL == details)
@@ -85,7 +89,6 @@ TMH_PG_lookup_token_family_key (
       GNUNET_PQ_result_spec_end
     };
 
-    check_connection (pg);
     return GNUNET_PQ_eval_prepared_singleton_select (pg->conn,
                                                      "lookup_token_family_key",
                                                      params,
@@ -105,17 +108,23 @@ TMH_PG_lookup_token_family_key (
                                                &details->priv.private_key),
         NULL),
       GNUNET_PQ_result_spec_allow_null (
-        GNUNET_PQ_result_spec_timestamp ("key_valid_after",
-                                         &details->valid_after),
+        GNUNET_PQ_result_spec_timestamp ("signature_validity_start",
+                                         &details->signature_validity_start),
         NULL),
       GNUNET_PQ_result_spec_allow_null (
-        GNUNET_PQ_result_spec_timestamp ("key_valid_before",
-                                         &details->valid_before),
+        GNUNET_PQ_result_spec_timestamp ("signature_validity_end",
+                                         &details->signature_validity_end),
+        NULL),
+      GNUNET_PQ_result_spec_allow_null (
+        GNUNET_PQ_result_spec_timestamp ("private_key_deleted_at",
+                                         &details->private_key_deleted_at),
         NULL),
       GNUNET_PQ_result_spec_string ("slug",
                                     &details->token_family.slug),
       GNUNET_PQ_result_spec_string ("name",
                                     &details->token_family.name),
+      GNUNET_PQ_result_spec_string ("cipher_choice",
+                                    &details->token_family.cipher_spec),
       GNUNET_PQ_result_spec_string ("description",
                                     &details->token_family.description),
       TALER_PQ_result_spec_json ("description_i18n",
@@ -126,6 +135,11 @@ TMH_PG_lookup_token_family_key (
                                        &details->token_family.valid_before),
       GNUNET_PQ_result_spec_relative_time ("duration",
                                            &details->token_family.duration),
+      GNUNET_PQ_result_spec_relative_time ("validity_granularity",
+                                           &details->token_family.
+                                           validity_granularity),
+      GNUNET_PQ_result_spec_relative_time ("start_offset",
+                                           &details->token_family.start_offset),
       GNUNET_PQ_result_spec_string ("kind",
                                     &kind),
       GNUNET_PQ_result_spec_uint64 ("issued",
