@@ -42,7 +42,7 @@ TMH_PG_select_account (void *cls,
     GNUNET_PQ_result_spec_auto_from_type ("salt",
                                           &ad->salt),
     GNUNET_PQ_result_spec_string ("payto_uri",
-                                  &ad->payto_uri),
+                                  &ad->payto_uri.full_payto),
     GNUNET_PQ_result_spec_allow_null (
       GNUNET_PQ_result_spec_string ("credit_facade_url",
                                     &ad->credit_facade_url),
@@ -57,6 +57,7 @@ TMH_PG_select_account (void *cls,
   };
 
   ad->h_wire = *h_wire;
+  ad->instance_id = id;
   check_connection (pg);
   PREPARE (pg,
            "select_account",
