@@ -32,8 +32,11 @@
 #include "taler_merchantdb_plugin.h"
 #include "pg_helper.h"
 #include "pg_insert_otp.h"
+#include "pg_get_kyc_status.h"
+#include "pg_get_kyc_limits.h"
 #include "pg_delete_otp.h"
 #include "pg_update_otp.h"
+#include "pg_account_kyc_set_failed.h"
 #include "pg_select_otp.h"
 #include "pg_select_otp_serial.h"
 #include "pg_insert_login_token.h"
@@ -74,6 +77,7 @@
 #include "pg_delete_product.h"
 #include "pg_insert_product.h"
 #include "pg_update_product.h"
+#include "pg_lookup_token_family_keys.h"
 #include "pg_lock_product.h"
 #include "pg_expire_locks.h"
 #include "pg_delete_order.h"
@@ -317,11 +321,12 @@ postgres_connect (void *cls)
     GNUNET_PQ_EXECUTE_STATEMENT_END
   };
 
-  pg->conn = GNUNET_PQ_connect_with_cfg (pg->cfg,
-                                         "merchantdb-postgres",
-                                         NULL,
-                                         es,
-                                         NULL);
+  pg->conn = GNUNET_PQ_connect_with_cfg2 (pg->cfg,
+                                          "merchantdb-postgres",
+                                          "merchant-",
+                                          es,
+                                          NULL, /* prepared statemetns */
+                                          GNUNET_PQ_FLAG_CHECK_CURRENT);
   pg->prep_gen++;
   if (NULL == pg->conn)
     return GNUNET_SYSERR;
@@ -336,6 +341,10 @@ postgres_connect (void *cls)
  * @return NULL on error, otherwise a `struct TALER_MERCHANTDB_Plugin`
  */
 void *
+libtaler_plugin_merchantdb_postgres_init (void *cls);
+
+/* declaration to avoid compiler warning */
+void *
 libtaler_plugin_merchantdb_postgres_init (void *cls)
 {
   const struct GNUNET_CONFIGURATION_Handle *cfg = cls;
@@ -543,6 +552,8 @@ libtaler_plugin_merchantdb_postgres_init (void *cls)
     = &TMH_PG_insert_deposit_to_transfer;
   plugin->insert_transfer
     = &TMH_PG_insert_transfer;
+  plugin->lookup_token_family_keys
+    = &TMH_PG_lookup_token_family_keys;
   plugin->insert_transfer_details
     = &TMH_PG_insert_transfer_details;
   plugin->store_wire_fee_by_exchange
@@ -577,6 +588,12 @@ libtaler_plugin_merchantdb_postgres_init (void *cls)
     = &TMH_PG_lookup_categories;
   plugin->select_category_by_name
     = &TMH_PG_select_category_by_name;
+  plugin->get_kyc_status
+    = &TMH_PG_get_kyc_status;
+  plugin->account_kyc_set_failed
+    = &TMH_PG_account_kyc_set_failed;
+  plugin->get_kyc_limits
+    = &TMH_PG_get_kyc_limits;
   plugin->select_category
     = &TMH_PG_select_category;
   plugin->update_category
@@ -615,8 +632,6 @@ libtaler_plugin_merchantdb_postgres_init (void *cls)
     = &TMH_PG_lookup_spent_tokens_by_order;
 
 
-
-
   return plugin;
 }
 
@@ -628,6 +643,10 @@ libtaler_plugin_merchantdb_postgres_init (void *cls)
  * @return NULL (always)
  */
 void *
+libtaler_plugin_merchantdb_postgres_done (void *cls);
+
+/* declaration to avoid compiler warning */
+void *
 libtaler_plugin_merchantdb_postgres_done (void *cls)
 {
   struct TALER_MERCHANTDB_Plugin *plugin = cls;
