@@ -20,6 +20,10 @@ DECLARE
   my_decose INT8;
 DECLARE
   my_order_serial INT8;
+DECLARE
+  my_merchant_serial INT8;
+DECLARE
+  my_order_id TEXT;
 BEGIN
 SELECT signkey_serial
   INTO my_signkey_serial
@@ -268,6 +272,8 @@ DECLARE
   my_signkey_serial INT8;
   my_credit_serial INT8;
   my_affected_orders RECORD;
+  my_merchant_serial INT8;
+  my_order_id TEXT;
   i INT8;
   curs CURSOR (arg_coin_pub BYTEA) FOR
     SELECT mcon.deposit_confirmation_serial,
@@ -418,6 +424,24 @@ LOOP
             FROM merchant_deposit_confirmations mdc
              WHERE mdc.wire_pending
                AND mdc.order_serial=my_affected_orders.order_serial);
+      SELECT merchant_serial, order_id
+        INTO my_merchant_serial, my_order_id
+        FROM merchant_contract_terms
+       WHERE order_serial=my_affected_orders.order_serial;
+      INSERT INTO merchant_pending_webhooks
+      (merchant_serial
+      ,webhook_serial
+      ,url
+      ,http_method
+      ,body)
+      SELECT mw.merchant_serial
+            ,mw.webhook_serial
+            ,mw.url
+            ,mw.http_method
+            ,replace_placeholder(mw.body_template, 'order_id', my_order_id)::TEXT
+      FROM merchant_webhook mw
+      WHERE mw.event_type = 'order_settled'
+        AND mw.merchant_serial = my_merchant_serial;
     END IF;
   END LOOP; 
   CLOSE curs;
@@ -525,20 +549,20 @@ UPDATE merchant_inventory SET
    AND product_serial=my_product_serial; 
 ASSERT FOUND,'SELECTED it earlier, should UPDATE it now';
 END $$;
-CREATE OR REPLACE FUNCTION merchant_do_account_kyc_set_status (
+DROP FUNCTION IF EXISTS merchant_do_account_kyc_set_status;
+CREATE FUNCTION merchant_do_account_kyc_set_status (
   IN in_merchant_id TEXT,
   IN in_h_wire BYTEA,
   IN in_exchange_url TEXT,
   IN in_timestamp INT8,
-  IN in_exchange_kyc_serial INT8,
   IN in_exchange_http_status INT4,
   IN in_exchange_ec_code INT4,
-  IN in_access_token BYTEA,
-  IN ina_thresholds taler_amount_currency[],
-  IN ina_timeframes INT8[],
-  IN ina_soft_limits BOOL[],
+  IN in_access_token BYTEA, 
+  IN in_jlimits TEXT,
   IN in_aml_active BOOL,
   IN in_kyc_ok BOOL,
+  IN in_notify_str TEXT,
+  IN in_notify2_str TEXT,
   OUT out_no_instance BOOL,
   OUT out_no_account BOOL)
 LANGUAGE plpgsql
@@ -546,8 +570,6 @@ AS $$
 DECLARE
   my_merchant_id INT8;
   my_account_serial INT8;
-  ini_cat INT8;
-  rec RECORD;
 BEGIN
 out_no_instance=FALSE;
 out_no_account=FALSE;
@@ -570,48 +592,112 @@ THEN
   out_no_account=TRUE;
   RETURN;
 END IF;
-INSERT INTO merchant_kyc
-  (kyc_timestamp
-  ,kyc_ok
-  ,exchange_kyc_serial
-  ,account_serial
-  ,exchange_url
-  ,deposit_thresholds
-  ,deposit_timeframes
-  ,deposit_limits_are_soft
-  ,aml_review
-  ,exchange_http_status
-  ,exchange_ec_code
-  ,access_token)
-VALUES
-  (in_timestamp
-  ,in_kyc_ok
-  ,in_exchange_kyc_serial
-  ,my_account_serial
-  ,in_exchange_url
-  ,ina_thresholds
-  ,ina_timeframes
-  ,ina_soft_limits
-  ,in_aml_active
-  ,in_exchange_http_status
-  ,in_exchange_ec_code
-  ,in_access_token)
-  ON CONFLICT DO NOTHING;
+UPDATE merchant_kyc
+   SET kyc_timestamp=in_timestamp
+      ,kyc_ok=in_kyc_ok
+      ,jaccount_limits=in_jlimits
+      ,aml_review=in_aml_active
+      ,exchange_http_status=in_exchange_http_status
+      ,exchange_ec_code=in_exchange_ec_code
+      ,access_token=in_access_token
+ WHERE account_serial=my_account_serial
+   AND exchange_url=in_exchange_url;
 IF NOT FOUND
 THEN
-  UPDATE merchant_kyc
-     SET exchange_kyc_serial=in_exchange_kyc_serial
-        ,kyc_timestamp=in_timestamp
-        ,kyc_ok=in_kyc_ok
-        ,deposit_thresholds=ina_thresholds
-        ,deposit_timeframes=ina_timeframes
-        ,deposit_limits_are_soft=ina_soft_limits
-        ,aml_review=in_aml_active
-        ,exchange_http_status=in_exchange_http_status
-        ,exchange_ec_code=in_exchange_ec_code
-        ,access_token=in_access_token
-   WHERE account_serial=my_account_serial
-     AND exchange_url=in_exchange_url;
+  INSERT INTO merchant_kyc
+    (kyc_timestamp
+    ,kyc_ok
+    ,account_serial
+    ,exchange_url
+    ,jaccount_limits
+    ,aml_review
+    ,exchange_http_status
+    ,exchange_ec_code
+    ,access_token)
+  VALUES
+    (in_timestamp
+    ,in_kyc_ok
+    ,my_account_serial
+    ,in_exchange_url
+    ,in_jlimits
+    ,in_aml_active
+    ,in_exchange_http_status
+    ,in_exchange_ec_code
+    ,in_access_token);
 END IF;
+EXECUTE FORMAT (
+   'NOTIFY %s'
+  ,in_notify_str);
+EXECUTE FORMAT (
+   'NOTIFY %s'
+  ,in_notify2_str);
+END $$;
+DROP FUNCTION IF EXISTS merchant_do_account_kyc_set_failed;
+CREATE FUNCTION merchant_do_account_kyc_set_failed (
+  IN in_merchant_id TEXT,
+  IN in_h_wire BYTEA,
+  IN in_exchange_url TEXT,
+  IN in_timestamp INT8,
+  IN in_exchange_http_status INT4,
+  IN in_kyc_ok BOOL,
+  IN in_notify_str TEXT,
+  IN in_notify2_str TEXT,
+  OUT out_no_instance BOOL,
+  OUT out_no_account BOOL)
+LANGUAGE plpgsql
+AS $$
+DECLARE
+  my_merchant_id INT8;
+  my_account_serial INT8;
+BEGIN
+out_no_instance=FALSE;
+out_no_account=FALSE;
+SELECT merchant_serial
+  INTO my_merchant_id
+  FROM merchant_instances
+ WHERE merchant_id=in_merchant_id;
+IF NOT FOUND
+THEN
+  out_no_instance=TRUE;
+  RETURN;
+END IF;
+SELECT account_serial
+  INTO my_account_serial
+  FROM merchant_accounts
+ WHERE merchant_serial=my_merchant_id
+   AND h_wire=in_h_wire;
+IF NOT FOUND
+THEN
+  out_no_account=TRUE;
+  RETURN;
+END IF;
+UPDATE merchant_kyc
+   SET kyc_timestamp=in_timestamp
+      ,kyc_ok=in_kyc_ok
+      ,exchange_http_status=in_exchange_http_status
+      ,exchange_ec_code=0
+ WHERE account_serial=my_account_serial
+   AND exchange_url=in_exchange_url;
+IF NOT FOUND
+THEN
+  INSERT INTO merchant_kyc
+    (kyc_timestamp
+    ,kyc_ok
+    ,account_serial
+    ,exchange_url
+    ,exchange_http_status)
+  VALUES
+    (in_timestamp
+    ,in_kyc_ok
+    ,my_account_serial
+    ,in_exchange_url
+    ,in_exchange_http_status);
+END IF;
+EXECUTE FORMAT (
+   'NOTIFY %s'
+  ,in_notify_str);
+EXECUTE FORMAT (
+   'NOTIFY %s'
+  ,in_notify2_str);
 END $$;
 COMMIT;
