@@ -181,6 +181,7 @@ TALER_MERCHANT_instance_get (struct GNUNET_CURL_Context *ctx,
   igh->ctx = ctx;
   igh->cb = cb;
   igh->cb_cls = cb_cls;
+  if (NULL != instance_id)
   {
     char *path;
 
@@ -192,6 +193,12 @@ TALER_MERCHANT_instance_get (struct GNUNET_CURL_Context *ctx,
                                NULL);
     GNUNET_free (path);
   }
+  else
+  {
+    igh->url = TALER_url_join (backend_url,
+                               "private",
+                               NULL);
+  }
   if (NULL == igh->url)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
