@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2023 Taler Systems SA
+  Copyright (C) 2014-2024 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Lesser General Public License as published by the Free Software
@@ -105,12 +105,16 @@ handle_get_product_finished (void *cls,
         TALER_JSON_spec_amount_any (
           "price",
           &pgr.details.ok.price),
-        GNUNET_JSON_spec_string (
-          "image",
-          &pgr.details.ok.image),
-        GNUNET_JSON_spec_array_const (
-          "taxes",
-          &pgr.details.ok.taxes),
+        GNUNET_JSON_spec_mark_optional (
+          GNUNET_JSON_spec_string (
+            "image",
+            &pgr.details.ok.image),
+          NULL),
+        GNUNET_JSON_spec_mark_optional (
+          GNUNET_JSON_spec_array_const (
+            "taxes",
+            &pgr.details.ok.taxes),
+          NULL),
         GNUNET_JSON_spec_int64 (
           "total_stock",
           &pgr.details.ok.total_stock),
@@ -120,12 +124,15 @@ handle_get_product_finished (void *cls,
         GNUNET_JSON_spec_uint64 (
           "total_lost",
           &pgr.details.ok.total_lost),
-        GNUNET_JSON_spec_object_const (
-          "address",
-          &pgr.details.ok.location),
         GNUNET_JSON_spec_mark_optional (
-          GNUNET_JSON_spec_timestamp ("next_restock",
-                                      &pgr.details.ok.next_restock),
+          GNUNET_JSON_spec_object_const (
+            "address",
+            &pgr.details.ok.location),
+          NULL),
+        GNUNET_JSON_spec_mark_optional (
+          GNUNET_JSON_spec_timestamp (
+            "next_restock",
+            &pgr.details.ok.next_restock),
           NULL),
         GNUNET_JSON_spec_end ()
       };
