@@ -130,8 +130,8 @@ handle_transfers_get_finished (void *cls,
                                         &td->credit_amount),
             GNUNET_JSON_spec_fixed_auto ("wtid",
                                          &td->wtid),
-            TALER_JSON_spec_payto_uri ("payto_uri",
-                                       &td->payto_uri),
+            TALER_JSON_spec_full_payto_uri ("payto_uri",
+                                            &td->payto_uri),
             TALER_JSON_spec_web_url ("exchange_url",
                                      &td->exchange_url),
             GNUNET_JSON_spec_uint64 ("transfer_serial_id",
@@ -219,7 +219,7 @@ struct TALER_MERCHANT_GetTransfersHandle *
 TALER_MERCHANT_transfers_get (
   struct GNUNET_CURL_Context *ctx,
   const char *backend_url,
-  const char *payto_uri,
+  struct TALER_FullPayto payto_uri,
   const struct GNUNET_TIME_Timestamp before,
   const struct GNUNET_TIME_Timestamp after,
   int64_t limit,
@@ -233,8 +233,8 @@ TALER_MERCHANT_transfers_get (
   const char *verified_s = NULL;
   char limit_s[30];
   char offset_s[30];
-  char *before_s;
-  char *after_s;
+  char before_s[30];
+  char after_s[30];
 
   gth = GNUNET_new (struct TALER_MERCHANT_GetTransfersHandle);
   gth->ctx = ctx;
@@ -249,35 +249,46 @@ TALER_MERCHANT_transfers_get (
                    sizeof (offset_s),
                    "%lld",
                    (unsigned long long) offset);
-  before_s = GNUNET_strdup (GNUNET_TIME_timestamp2s (before));
-  after_s = GNUNET_strdup (GNUNET_TIME_timestamp2s (after));
-  gth->url = TALER_url_join (backend_url,
-                             "private/transfers",
-                             "payto_uri",
-                             payto_uri,
-                             "verified",
-                             (TALER_EXCHANGE_YNA_ALL != verified)
-                             ? verified_s
-                             : NULL,
-                             "limit",
-                             0 != limit
-                             ? limit_s
-                             : NULL,
-                             "offset",
-                             ((0 != offset) && (UINT64_MAX != offset))
-                             ? offset_s
-                             : NULL,
-                             "before",
-                             GNUNET_TIME_absolute_is_never (before.abs_time)
-                             ? NULL
-                             : before_s,
-                             "after",
-                             GNUNET_TIME_absolute_is_zero (after.abs_time)
-                             ? NULL
-                             : after_s,
-                             NULL);
-  GNUNET_free (before_s);
-  GNUNET_free (after_s);
+
+
+  GNUNET_snprintf (before_s,
+                   sizeof (before_s),
+                   "%llu",
+                   (unsigned long long) GNUNET_TIME_timestamp_to_s (before));
+  GNUNET_snprintf (after_s,
+                   sizeof (after_s),
+                   "%llu",
+                   (unsigned long long) GNUNET_TIME_timestamp_to_s (after));
+  {
+    char *enc_payto = TALER_urlencode (payto_uri.full_payto);
+
+    gth->url = TALER_url_join (backend_url,
+                               "private/transfers",
+                               "payto_uri",
+                               enc_payto,
+                               "verified",
+                               (TALER_EXCHANGE_YNA_ALL != verified)
+                               ? verified_s
+                               : NULL,
+                               "limit",
+                               0 != limit
+                               ? limit_s
+                               : NULL,
+                               "offset",
+                               ((0 != offset) && (UINT64_MAX != offset))
+                               ? offset_s
+                               : NULL,
+                               "before",
+                               GNUNET_TIME_absolute_is_never (before.abs_time)
+                               ? NULL
+                               : before_s,
+                               "after",
+                               GNUNET_TIME_absolute_is_zero (after.abs_time)
+                               ? NULL
+                               : after_s,
+                               NULL);
+    GNUNET_free (enc_payto);
+  }
   if (NULL == gth->url)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
