@@ -144,20 +144,20 @@ parse_tokens (const json_t *token_sigs,
   for (unsigned int i = 0; i<(*num_tokens); i++)
   {
     struct TALER_MERCHANT_OutputToken *token = &(*tokens)[i];
-    const json_t *jtoken = json_array_get (token_sigs,
-                                       i);
+    struct GNUNET_JSON_Specification spec[] = {
+      TALER_JSON_spec_blinded_token_issue_sig ("blind_sig",
+                                               &token->blinded_sig),
+      GNUNET_JSON_spec_end ()
+    };
+    const json_t *jtoken
+      = json_array_get (token_sigs,
+                        i);
 
     if (NULL == jtoken)
     {
       GNUNET_break (0);
       return GNUNET_SYSERR;
     }
-    struct GNUNET_JSON_Specification spec[] = {
-      TALER_JSON_spec_blinded_token_issue_sig ("blind_sig",
-                                               &token->blinded_sig),
-      GNUNET_JSON_spec_end ()
-    };
-
     if (GNUNET_OK !=
         GNUNET_JSON_parse (jtoken,
                            spec,
@@ -285,9 +285,6 @@ handle_pay_finished (void *cls,
   case MHD_HTTP_FORBIDDEN:
     pr.hr.ec = TALER_JSON_get_error_code (json);
     pr.hr.hint = TALER_JSON_get_error_hint (json);
-    /* Nothing really to verify, merchant says we tried to abort the payment
-     * after it was successful. We should pass the JSON reply to the
-     * application */
     break;
   case MHD_HTTP_NOT_FOUND:
     pr.hr.ec = TALER_JSON_get_error_code (json);
@@ -325,6 +322,52 @@ handle_pay_finished (void *cls,
        of some disagreement on the PKI).  We should pass the JSON reply to the
        application */
     break;
+  case MHD_HTTP_UNAVAILABLE_FOR_LEGAL_REASONS:
+    {
+      json_t *ebus = json_object_get (json,
+                                      "exchange_base_urls");
+      if (NULL == ebus)
+      {
+        GNUNET_break_op (0);
+        pr.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+        pr.hr.http_status = 0;
+        pr.hr.hint = "failed to parse exchange_base_urls field in response";
+        break;
+      }
+      {
+        size_t alen = json_array_size (ebus);
+        const char *ebua[GNUNET_NZL (alen)];
+        size_t idx;
+        json_t *jebu;
+        bool ok = true;
+
+        GNUNET_assert (alen <= UINT_MAX);
+        json_array_foreach (ebus, idx, jebu)
+        {
+          ebua[idx] = json_string_value (jebu);
+          if (NULL == ebua[idx])
+          {
+            GNUNET_break_op (0);
+            pr.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+            pr.hr.http_status = 0;
+            pr.hr.hint = "non-string value in exchange_base_urls in response";
+            ok = false;
+            break;
+          }
+        }
+        if (! ok)
+          break;
+        pr.details.unavailable_for_legal_reasons.num_exchanges
+          = (unsigned int) alen;
+        pr.details.unavailable_for_legal_reasons.exchanges
+          = ebua;
+        oph->pay_cb (oph->pay_cb_cls,
+                     &pr);
+        TALER_MERCHANT_order_pay_cancel (oph);
+        return;
+      }
+    }
+    break;
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
     TALER_MERCHANT_parse_error_details_ (json,
                                          response_code,
@@ -477,11 +520,13 @@ TALER_MERCHANT_order_pay_frontend (
                                     &ut->token_sig),
         GNUNET_JSON_pack_data_auto ("token_pub",
                                     &ut->token_pub),
+        GNUNET_JSON_pack_data_auto ("h_issue",
+                                    &ut->issue_pub.public_key->pub_key_hash),
         TALER_JSON_pack_token_issue_sig ("ub_sig",
-                                        &ut->ub_sig));
+                                         &ut->ub_sig));
       if (0 !=
           json_array_append_new (j_tokens,
-                                j_token))
+                                 j_token))
       {
         GNUNET_break (0);
         json_decref (j_tokens);
@@ -581,7 +626,8 @@ TALER_MERCHANT_order_pay (
   unsigned int num_tokens,
   const struct TALER_MERCHANT_UseToken tokens[static num_tokens],
   unsigned int num_output_tokens,
-  const struct TALER_MERCHANT_OutputToken output_tokens[static num_output_tokens],
+  const struct TALER_MERCHANT_OutputToken output_tokens[static num_output_tokens
+  ],
   TALER_MERCHANT_OrderPayCallback pay_cb,
   void *pay_cb_cls)
 {
@@ -619,7 +665,7 @@ TALER_MERCHANT_order_pay (
 
       if (0 !=
           json_array_append_new (j_output_tokens,
-                                j_token_ev))
+                                 j_token_ev))
       {
         GNUNET_break (0);
         json_decref (j_output_tokens);
@@ -634,9 +680,9 @@ TALER_MERCHANT_order_pay (
                               choice_index),
       GNUNET_JSON_pack_allow_null (
         GNUNET_JSON_pack_array_incref ("tokens_evs",
-                                      j_output_tokens)));
+                                       j_output_tokens)));
     TALER_json_hash (wallet_data,
-                    &wallet_data_hash);
+                     &wallet_data_hash);
   }
   {
     struct TALER_MERCHANT_PaidCoin pc[num_coins];
