@@ -25,6 +25,7 @@
 #include "platform.h"
 #include <taler/taler_util.h>
 #include <taler/taler_testing_lib.h>
+#include "taler_merchant_util.h"
 #include "taler_merchant_testing_lib.h"
 
 
@@ -407,6 +408,7 @@ main (int argc,
       &use_fakebank),
     GNUNET_GETOPT_option_version (PACKAGE_VERSION " " VCS_VERSION),
     GNUNET_GETOPT_option_help (
+      TALER_MERCHANT_project_data (),
       "Runs benchmark logic against merchant backend. "
       "Must be used with either 'ordinary' or 'corner' sub-commands."),
     GNUNET_GETOPT_option_string (
@@ -431,7 +433,9 @@ main (int argc,
       "fakebank",
       "use fakebank for the banking system",
       &use_fakebank),
-    GNUNET_GETOPT_option_help ("Populate databases with corner case payments"),
+    GNUNET_GETOPT_option_help (
+      TALER_MERCHANT_project_data (),
+      "Populate databases with corner case payments"),
     GNUNET_GETOPT_option_string (
       'l',
       "logfile",
@@ -480,6 +484,7 @@ main (int argc,
       "use fakebank for the banking system",
       &use_fakebank),
     GNUNET_GETOPT_option_help (
+      TALER_MERCHANT_project_data (),
       "Generate Taler ordinary payments"
       " to populate the databases"),
     GNUNET_GETOPT_option_string (
@@ -500,7 +505,7 @@ main (int argc,
   };
   const char *default_config_file;
 
-  default_config_file = GNUNET_OS_project_data_get ()->user_config_file;
+  default_config_file = TALER_MERCHANT_project_data ()->user_config_file;
   options = root_options;
   if (NULL != argv[1])
   {
@@ -534,12 +539,13 @@ main (int argc,
     }
   }
   if (NULL == exchange_bank_section)
-    exchange_bank_section = "exchange-account-1";
+    exchange_bank_section = GNUNET_strdup ("exchange-account-1");
   if (NULL == loglev)
-    loglev = "INFO";
+    loglev = GNUNET_strdup ("INFO");
   GNUNET_log_setup ("taler-merchant-benchmark",
                     loglev,
                     logfile);
+  GNUNET_free (loglev);
   if ( (! ordinary) &&
        (! corner) )
   {
@@ -552,7 +558,7 @@ main (int argc,
   {
     struct GNUNET_CONFIGURATION_Handle *cfg;
 
-    cfg = GNUNET_CONFIGURATION_create ();
+    cfg = GNUNET_CONFIGURATION_create (TALER_MERCHANT_project_data ());
     if (GNUNET_OK !=
         GNUNET_CONFIGURATION_load (cfg,
                                    cfg_filename))
@@ -562,6 +568,7 @@ main (int argc,
     }
     if (GNUNET_OK !=
         TALER_config_get_currency (cfg,
+                                   "merchant",
                                    &currency))
     {
       TALER_LOG_ERROR ("Failed to read currency from configuration\n");
