@@ -22,6 +22,7 @@
 #include <taler/taler_util.h>
 #include <taler/taler_dbevents.h>
 #include <gnunet/gnunet_util_lib.h>
+#include "taler_merchant_util.h"
 #include "taler_merchantdb_lib.h"
 #include "taler_merchantdb_lib.h"
 
@@ -105,7 +106,7 @@ run (void *cls,
       plugin->connect (plugin->cls))
   {
     fprintf (stderr,
-             "Failed to connect to database\n");
+             "Failed to connect to database. Consider running taler-merchant-dbinit!\n");
     global_ret = 1;
     TALER_MERCHANTDB_plugin_unload (plugin);
     GNUNET_CONFIGURATION_destroy (cfg);
@@ -171,21 +172,13 @@ main (int argc,
   };
   enum GNUNET_GenericReturnValue ret;
 
-  /* force linker to link against libtalerutil; if we do
-     not do this, the linker may "optimize" libtalerutil
-     away and skip #TALER_OS_init(), which we do need */
-  (void) TALER_project_data_default ();
-  if (GNUNET_OK !=
-      GNUNET_STRINGS_get_utf8_args (argc, argv,
-                                    &argc, &argv))
-    return 4;
   ret = GNUNET_PROGRAM_run (
+    TALER_MERCHANT_project_data (),
     argc, argv,
     "taler-merchant-passwd",
     gettext_noop ("Reset instance password"),
     options,
     &run, NULL);
-  GNUNET_free_nz ((void *) argv);
   if (GNUNET_SYSERR == ret)
     return 3;
   if (GNUNET_NO == ret)
