@@ -36,7 +36,7 @@ taler-harness --help >/dev/null </dev/null || exit_skip " MISSING"
 echo " FOUND"
 
 
-export WALLET_HARNESS_WITH_EUFIN=1
+export WITH_LIBEUFIN=1
 res=0
 taler-harness run-integrationtests --dry --suites merchant 2&>/dev/null || res=$?
 
