@@ -51,17 +51,17 @@
 /**
  * Payto URI of the customer (payer).
  */
-static const char *payer_payto;
+static struct TALER_FullPayto payer_payto;
 
 /**
  * Payto URI of the exchange (escrow account).
  */
-static const char *exchange_payto;
+static struct TALER_FullPayto exchange_payto;
 
 /**
  * Payto URI of the merchant (receiver).
  */
-static const char *merchant_payto;
+static struct TALER_FullPayto merchant_payto;
 
 /**
  * Credentials for the test.
@@ -191,6 +191,48 @@ run (void *cls,
       "create-reserve-1",
       "EUR:0",
       MHD_HTTP_OK),
+
+    /* KYC: we don't even know the legitimization UUID yet */
+    TALER_TESTING_cmd_merchant_kyc_get (
+      "kyc-pending-early",
+      merchant_url,
+      NULL,
+      NULL,
+      EXCHANGE_URL,
+      TALER_EXCHANGE_KLPT_NONE,
+      MHD_HTTP_OK,
+      true),
+    /* now we get the legi UUID by running taler-merchant-depositcheck */
+    TALER_TESTING_cmd_depositcheck (
+      "deposit-check",
+      CONFIG_FILE),
+
+    TALER_TESTING_cmd_merchant_get_instance (
+      "get-default-instance",
+      merchant_url,
+      NULL,
+      MHD_HTTP_OK,
+      "instance-create-default-setup"),
+    TALER_TESTING_cmd_admin_add_kycauth (
+      "merchant-kyc-auth-transfer",
+      "EUR:0.01",
+      &cred.ba,
+      merchant_payto,
+      "get-default-instance"),
+    CMD_EXEC_WIREWATCH (
+      "import-kyc-account-withdraw"),
+
+    TALER_TESTING_cmd_merchant_kyc_get (
+      "kyc-auth-done-check",
+      merchant_url,
+      NULL, /* default instance */
+      "instance-create-default-account", /* h_wire_ref: which account to query */
+      EXCHANGE_URL,
+      TALER_EXCHANGE_KLPT_KYC_AUTH_TRANSFER,
+      MHD_HTTP_OK,
+      true),
+
+
     TALER_TESTING_cmd_merchant_post_orders2 (
       "create-proposal-1",
       cred.cfg,
@@ -211,6 +253,7 @@ run (void *cls,
       MHD_HTTP_OK,
       "create-proposal-1",
       NULL),
+
     TALER_TESTING_cmd_merchant_pay_order (
       "deposit-simple",
       merchant_url,
@@ -233,43 +276,14 @@ run (void *cls,
     /* KYC: hence nothing happened at the bank yet: */
     TALER_TESTING_cmd_check_bank_empty (
       "check_bank_empty-2"),
-    /* KYC: we don't even know the legitimization UUID yet */
-    TALER_TESTING_cmd_merchant_kyc_get (
-      "kyc-pending-early",
-      merchant_url,
-      NULL,
-      NULL,
-      EXCHANGE_URL,
-      MHD_HTTP_NO_CONTENT,
-      false),
-    /* now we get the legi UUID by running taler-merchant-depositcheck */
-    TALER_TESTING_cmd_depositcheck (
-      "deposit-check",
-      CONFIG_FILE),
-    TALER_TESTING_cmd_merchant_get_instance (
-      "get-default-instance",
-      merchant_url,
-      NULL,
-      MHD_HTTP_OK,
-      "instance-create-default-setup"),
-    TALER_TESTING_cmd_admin_add_kycauth (
-      "merchant-kyc-auth-transfer",
-      "EUR:0.01",
-      &cred.ba,
-      merchant_payto,
-      "get-default-instance"),
-    CMD_EXEC_WIREWATCH (
-      "import-kyc-account-withdraw"),
-
-
-    /* Now we should get a status of pending */
     TALER_TESTING_cmd_merchant_kyc_get (
       "kyc-pending",
       merchant_url,
-      NULL, /* instance */
-      NULL, /* h_wire_ref: which account to query */
+      NULL, /* default instance */
+      "instance-create-default-account", /* h_wire_ref: which account to query */
       EXCHANGE_URL,
-      MHD_HTTP_ACCEPTED,
+      TALER_EXCHANGE_KLPT_KYC_AUTH_TRANSFER,
+      MHD_HTTP_OK,
       true),
     TALER_TESTING_cmd_get_kyc_info (
       "get-kyc-info-deposit",
@@ -303,7 +317,7 @@ run (void *cls,
       "deposit-simple",
       NULL),
     TALER_TESTING_cmd_run_tme (
-      "run taler-merchant-exchange-1",
+      "run taler-merchant-reconciliation-1",
       CONFIG_FILE),
     TALER_TESTING_cmd_merchant_get_transfers (
       "get-transfers-1",
@@ -329,6 +343,9 @@ run (void *cls,
     TALER_TESTING_cmd_exec_wirewatch (
       "wirewatch-big",
       CONFIG_FILE),
+    TALER_TESTING_cmd_sleep (
+      "sleep to de-collide AML timestamps",
+      1),
     TALER_TESTING_cmd_take_aml_decision (
       "freeze",
       "aml-officer",
@@ -426,7 +443,8 @@ run (void *cls,
       NULL,     /* no instance ID */
       NULL,     /* no wire ref */
       EXCHANGE_URL,
-      MHD_HTTP_ACCEPTED,
+      TALER_EXCHANGE_KLPT_KYC_AUTH_TRANSFER,
+      MHD_HTTP_OK,
       true),
     TALER_TESTING_cmd_sleep (
       "sleep to de-collide AML timestamps",
@@ -454,8 +472,9 @@ run (void *cls,
       NULL,     /* no instance ID */
       NULL,     /* no wire ref */
       EXCHANGE_URL,
-      MHD_HTTP_NO_CONTENT,
-      false),
+      TALER_EXCHANGE_KLPT_KYC_OK,
+      MHD_HTTP_OK,
+      true),
     CMD_EXEC_AGGREGATOR ("run-aggregator-aml-normal"),
     TALER_TESTING_cmd_check_bank_transfer (
       "check_bank_transfer-498c-post-unfreeze",
@@ -473,7 +492,7 @@ run (void *cls,
       "deposit-simple",
       NULL),
     TALER_TESTING_cmd_run_tme (
-      "run taler-merchant-exchange-2-aml",
+      "run taler-merchant-reconciliation-2-aml",
       CONFIG_FILE),
     TALER_TESTING_cmd_merchant_get_transfers (
       "get-transfers-aml",
@@ -497,6 +516,7 @@ run (void *cls,
       CONFIG_FILE,
       "-ema",
       "-u", "exchange-account-exchange",
+      "-r", "merchant-exchange-test",
       NULL),
     TALER_TESTING_cmd_get_exchange (
       "get-exchange",
@@ -521,10 +541,8 @@ run (void *cls,
       MHD_HTTP_OK),
     TALER_TESTING_cmd_batch ("pay",
                              pay),
-#if FIXME_FUTURE_WORK || 1
     TALER_TESTING_cmd_batch ("aml",
                              aml),
-#endif
     TALER_TESTING_cmd_end ()
   };
 
@@ -537,15 +555,15 @@ int
 main (int argc,
       char *const *argv)
 {
-  payer_payto =
-    "payto://x-taler-bank/localhost/" USER_ACCOUNT_NAME "?receiver-name="
-    USER_ACCOUNT_NAME;
-  exchange_payto =
-    "payto://x-taler-bank/localhost/" EXCHANGE_ACCOUNT_NAME "?receiver-name="
-    EXCHANGE_ACCOUNT_NAME;
-  merchant_payto =
-    "payto://x-taler-bank/localhost/" MERCHANT_ACCOUNT_NAME "?receiver-name="
-    MERCHANT_ACCOUNT_NAME;
+  payer_payto.full_payto =
+    (char *) "payto://x-taler-bank/localhost/" USER_ACCOUNT_NAME
+    "?receiver-name=" USER_ACCOUNT_NAME;
+  exchange_payto.full_payto =
+    (char *) "payto://x-taler-bank/localhost/" EXCHANGE_ACCOUNT_NAME
+    "?receiver-name=" EXCHANGE_ACCOUNT_NAME;
+  merchant_payto.full_payto =
+    (char *) "payto://x-taler-bank/localhost/" MERCHANT_ACCOUNT_NAME
+    "?receiver-name=" MERCHANT_ACCOUNT_NAME;
   merchant_url = "http://localhost:8080/";
   GNUNET_asprintf (&merchant_url_i1a,
                    "%sinstances/i1a/",
