@@ -60,8 +60,6 @@
  */
 static char *config_file;
 
-#define PAYTO_I1 "payto://x-taler-bank/localhost/3?receiver-name=3"
-
 /**
  * Exchange base URL.  Could also be taken from config.
  */
@@ -70,17 +68,17 @@ static char *config_file;
 /**
  * Payto URI of the customer (payer).
  */
-static const char *payer_payto;
+static struct TALER_FullPayto payer_payto;
 
 /**
  * Payto URI of the exchange (escrow account).
  */
-static const char *exchange_payto;
+static struct TALER_FullPayto exchange_payto;
 
 /**
  * Payto URI of the merchant (receiver).
  */
-static const char *merchant_payto;
+static struct TALER_FullPayto merchant_payto;
 
 /**
  * Credentials for the test.
@@ -216,7 +214,7 @@ run (void *cls,
     TALER_TESTING_cmd_merchant_post_account (
       "instance-create-default-account",
       merchant_url,
-      PAYTO_I1,
+      merchant_payto,
       NULL, NULL,
       MHD_HTTP_OK),
     TALER_TESTING_cmd_merchant_kyc_get (
@@ -225,12 +223,13 @@ run (void *cls,
       NULL,
       NULL,
       EXCHANGE_URL,
+      TALER_EXCHANGE_KLPT_NONE,
       MHD_HTTP_NO_CONTENT,
       false),
     TALER_TESTING_cmd_merchant_post_orders_no_claim (
       "create-proposal-bad-currency",
       merchant_url,
-      MHD_HTTP_BAD_REQUEST,
+      MHD_HTTP_CONFLICT,
       "4",
       GNUNET_TIME_UNIT_ZERO_TS,
       GNUNET_TIME_UNIT_FOREVER_TS,
@@ -489,17 +488,17 @@ run (void *cls,
                                            merchant_payto),
     TALER_TESTING_cmd_merchant_post_transfer ("post-transfer-1",
                                               &cred.ba,
-                                              PAYTO_I1,
+                                              merchant_payto,
                                               merchant_url,
                                               "EUR:4.98",
                                               MHD_HTTP_NO_CONTENT,
                                               "deposit-simple",
                                               NULL),
-    TALER_TESTING_cmd_run_tme ("run taler-merchant-exchange-1",
+    TALER_TESTING_cmd_run_tme ("run taler-merchant-reconciliation-1",
                                config_file),
     TALER_TESTING_cmd_merchant_post_transfer2 ("post-transfer-bad",
                                                merchant_url,
-                                               PAYTO_I1,
+                                               merchant_payto,
                                                "EUR:4.98",
                                                NULL,
                                                /*non-routable IP address
@@ -509,7 +508,7 @@ run (void *cls,
                                                MHD_HTTP_NO_CONTENT),
     TALER_TESTING_cmd_merchant_get_transfers ("get-transfers-1",
                                               merchant_url,
-                                              PAYTO_I1,
+                                              merchant_payto,
                                               MHD_HTTP_OK,
                                               "post-transfer-1",
                                               "post-transfer-bad",
@@ -976,7 +975,7 @@ run (void *cls,
     TALER_TESTING_cmd_merchant_post_account (
       "instance-create-i1a-account",
       merchant_url_i1a,
-      PAYTO_I1,
+      merchant_payto,
       NULL, NULL,
       MHD_HTTP_OK),
     TALER_TESTING_cmd_merchant_get_product ("get-nx-product-i1a-1",
@@ -1698,17 +1697,15 @@ run (void *cls,
       "An upcoming subscription that is not valid yet.",
       NULL,
       /* In one day */
-      GNUNET_TIME_absolute_to_timestamp
-      (
+      GNUNET_TIME_absolute_to_timestamp (
         GNUNET_TIME_absolute_add (
-          GNUNET_TIME_timestamp_get ()
-          .abs_time, GNUNET_TIME_UNIT_DAYS)),
+          GNUNET_TIME_timestamp_get ().abs_time,
+          GNUNET_TIME_UNIT_DAYS)),
       /* In a year */
-      GNUNET_TIME_absolute_to_timestamp
-      (
+      GNUNET_TIME_absolute_to_timestamp (
         GNUNET_TIME_absolute_add (
-          GNUNET_TIME_timestamp_get ()
-          .abs_time, GNUNET_TIME_UNIT_YEARS)),
+          GNUNET_TIME_timestamp_get ().abs_time,
+          GNUNET_TIME_UNIT_YEARS)),
       GNUNET_TIME_UNIT_MONTHS,
       GNUNET_TIME_UNIT_MONTHS,
       "subscription"),
@@ -1716,54 +1713,57 @@ run (void *cls,
       "create-order-with-upcoming-output",
       cred.cfg,
       merchant_url,
-      MHD_HTTP_CONFLICT,
-      "create-upcoming-tokenfamily",
+      MHD_HTTP_OK,
+      "subscription-upcoming",
       0,
       1,
       "5-upcoming-output",
       GNUNET_TIME_UNIT_ZERO_TS,
       GNUNET_TIME_UNIT_FOREVER_TS,
       "EUR:5.0"),
-    TALER_TESTING_cmd_merchant_post_tokenfamilies ("create-tokenfamily",
-                                                   merchant_url,
-                                                   MHD_HTTP_NO_CONTENT,
-                                                   "subscription-1",
-                                                   "Subscription",
-                                                   "A subscription.",
-                                                   NULL,
-                                                   GNUNET_TIME_UNIT_ZERO_TS,
-                                                   GNUNET_TIME_relative_to_timestamp
-                                                     (GNUNET_TIME_UNIT_YEARS),
-                                                   GNUNET_TIME_UNIT_MONTHS,
-                                                   GNUNET_TIME_UNIT_MONTHS,
-                                                   "subscription"),
-    TALER_TESTING_cmd_merchant_post_orders_choices ("create-order-with-output",
-                                                    cred.cfg,
-                                                    merchant_url,
-                                                    MHD_HTTP_OK,
-                                                    "create-tokenfamily",
-                                                    0,
-                                                    1,
-                                                    "5-output",
-                                                    GNUNET_TIME_UNIT_ZERO_TS,
-                                                    GNUNET_TIME_UNIT_FOREVER_TS,
-                                                    "EUR:5.0"),
-    TALER_TESTING_cmd_merchant_pay_order_choices ("pay-order-with-output",
-                                                  merchant_url,
-                                                  MHD_HTTP_OK,
-                                                  "create-order-with-output",
-                                                  "withdraw-coin-1",
-                                                  "EUR:5",
-                                                  "EUR:4.99",
-                                                  NULL,
-                                                  0,
-                                                  NULL),
+    TALER_TESTING_cmd_merchant_post_tokenfamilies (
+      "create-tokenfamily",
+      merchant_url,
+      MHD_HTTP_NO_CONTENT,
+      "subscription-1",
+      "Subscription",
+      "A subscription.",
+      NULL,
+      GNUNET_TIME_UNIT_ZERO_TS,
+      GNUNET_TIME_relative_to_timestamp
+        (GNUNET_TIME_UNIT_YEARS),
+      GNUNET_TIME_UNIT_MONTHS,
+      GNUNET_TIME_UNIT_MONTHS,
+      "subscription"),
+    TALER_TESTING_cmd_merchant_post_orders_choices (
+      "create-order-with-output",
+      cred.cfg,
+      merchant_url,
+      MHD_HTTP_OK,
+      "subscription-1",
+      0,
+      1,
+      "5-output",
+      GNUNET_TIME_UNIT_ZERO_TS,
+      GNUNET_TIME_UNIT_FOREVER_TS,
+      "EUR:5.0"),
+    TALER_TESTING_cmd_merchant_pay_order_choices (
+      "pay-order-with-output",
+      merchant_url,
+      MHD_HTTP_OK,
+      "create-order-with-output",
+      "withdraw-coin-1",
+      "EUR:5",
+      "EUR:4.99",
+      NULL,
+      0,
+      NULL),
     TALER_TESTING_cmd_merchant_post_orders_choices (
       "create-order-with-input-and-output",
       cred.cfg,
       merchant_url,
       MHD_HTTP_OK,
-      "create-tokenfamily",
+      "subscription-1",
       1,
       1,
       "5-input-output",
@@ -1796,7 +1796,7 @@ run (void *cls,
       cred.cfg,
       merchant_url,
       MHD_HTTP_OK,
-      "create-tokenfamily",
+      "subscription-1",
       1,
       1,
       "5-input-output-2",
@@ -1827,6 +1827,7 @@ run (void *cls,
       config_file,
       "-ema",
       "-u", "exchange-account-exchange",
+      "-r", "merchant-exchange-test",
       NULL),
     TALER_TESTING_cmd_get_exchange (
       "get-exchange",
@@ -1854,7 +1855,7 @@ run (void *cls,
     TALER_TESTING_cmd_merchant_post_account (
       "instance-create-default-account",
       merchant_url,
-      PAYTO_I1,
+      merchant_payto,
       NULL, NULL,
       MHD_HTTP_OK),
     TALER_TESTING_cmd_merchant_post_instances (
@@ -1988,7 +1989,7 @@ run (void *cls,
     TALER_TESTING_cmd_merchant_post_account (
       "instance-create-default-account-after-purge",
       merchant_url,
-      PAYTO_I1,
+      merchant_payto,
       NULL, NULL,
       MHD_HTTP_OK),
     TALER_TESTING_cmd_merchant_get_products (
@@ -2165,15 +2166,16 @@ main (int argc,
                      cipher);
     GNUNET_free (cipher);
   }
-  payer_payto =
-    "payto://x-taler-bank/localhost/" USER_ACCOUNT_NAME "?receiver-name="
-    USER_ACCOUNT_NAME;
-  exchange_payto =
-    "payto://x-taler-bank/localhost/" EXCHANGE_ACCOUNT_NAME "?receiver-name="
+  payer_payto.full_payto =
+    (char *) "payto://x-taler-bank/localhost/" USER_ACCOUNT_NAME
+    "?receiver-name=" USER_ACCOUNT_NAME;
+  exchange_payto.full_payto =
+    (char *) "payto://x-taler-bank/localhost/" EXCHANGE_ACCOUNT_NAME
+    "?receiver-name="
     EXCHANGE_ACCOUNT_NAME;
-  merchant_payto =
-    "payto://x-taler-bank/localhost/" MERCHANT_ACCOUNT_NAME "?receiver-name="
-    MERCHANT_ACCOUNT_NAME;
+  merchant_payto.full_payto =
+    (char *) "payto://x-taler-bank/localhost/" MERCHANT_ACCOUNT_NAME
+    "?receiver-name=" MERCHANT_ACCOUNT_NAME;
   merchant_url = "http://localhost:8080/";
   GNUNET_asprintf (&merchant_url_i1a,
                    "%sinstances/i1a/",
