@@ -61,23 +61,19 @@ static char *config_file;
  */
 #define USER_ACCOUNT_NAME "62"
 
-
-#define PAYTO_I1 "payto://x-taler-bank/localhost/3?receiver-name=user3"
-
-
 /**
  * Configuration file for the proxy between merchant and
  * exchange.  Not used directly here in the code (instead
  * used in the merchant config), but kept around for consistency.
  */
 #define PROXY_EXCHANGE_config_file \
-  "test_merchant_api_proxy_exchange.conf"
+        "test_merchant_api_proxy_exchange.conf"
 
 /**
  * Configuration file for the proxy between "lib" and merchant.
  */
 #define PROXY_MERCHANT_config_file \
-  "test_merchant_api_proxy_merchant.conf"
+        "test_merchant_api_proxy_merchant.conf"
 
 /**
  * Exchange base URL.  Could also be taken from config.
@@ -119,10 +115,10 @@ static struct GNUNET_OS_Process *twisterexchanged;
  */
 static struct GNUNET_OS_Process *twistermerchantd;
 
+static struct TALER_FullPayto payer_payto;
+static struct TALER_FullPayto exchange_payto;
+static struct TALER_FullPayto merchant_payto;
 
-static const char *payer_payto;
-static const char *exchange_payto;
-static const char *merchant_payto;
 static struct TALER_TESTING_Credentials cred;
 
 /**
@@ -155,8 +151,8 @@ CMD_EXEC_WIREWATCH (const char *label)
  * @param label label to use for the command.
  */
 #define CMD_EXEC_AGGREGATOR(label) \
-  TALER_TESTING_cmd_exec_aggregator (label "-aggregator", config_file), \
-  TALER_TESTING_cmd_exec_transfer (label "-transfer", config_file)
+        TALER_TESTING_cmd_exec_aggregator (label "-aggregator", config_file), \
+        TALER_TESTING_cmd_exec_transfer (label "-transfer", config_file)
 
 
 /**
@@ -328,6 +324,7 @@ run (void *cls,
                                     config_file,
                                     "-ema",
                                     "-u", "exchange-account-exchange",
+                                    "-r", "merchant-exchange-test",
                                     NULL),
     TALER_TESTING_cmd_get_exchange ("get-exchange",
                                     cred.cfg,
@@ -341,7 +338,7 @@ run (void *cls,
     TALER_TESTING_cmd_merchant_post_account (
       "instance-create-default-account",
       twister_merchant_url,
-      PAYTO_I1,
+      merchant_payto,
       NULL, NULL,
       MHD_HTTP_OK),
     TALER_TESTING_cmd_batch ("pay",
@@ -452,14 +449,15 @@ main (int argc,
                      cipher);
     GNUNET_free (cipher);
   }
-  payer_payto =
-    "payto://x-taler-bank/localhost/" USER_ACCOUNT_NAME "?receiver-name=user";
-  exchange_payto =
-    "payto://x-taler-bank/localhost/" EXCHANGE_ACCOUNT_NAME
-    "?receiver-name=exchange";
-  merchant_payto =
-    "payto://x-taler-bank/localhost/" MERCHANT_ACCOUNT_NAME
-    "?receiver-name=merchant";
+  payer_payto.full_payto =
+    (char *) "payto://x-taler-bank/localhost/" USER_ACCOUNT_NAME
+    "?receiver-name=" USER_ACCOUNT_NAME;
+  exchange_payto.full_payto =
+    (char *) "payto://x-taler-bank/localhost/" EXCHANGE_ACCOUNT_NAME
+    "?receiver-name=" EXCHANGE_ACCOUNT_NAME;
+  merchant_payto.full_payto =
+    (char *) "payto://x-taler-bank/localhost/" MERCHANT_ACCOUNT_NAME
+    "?receiver-name=" MERCHANT_ACCOUNT_NAME;
   merchant_url = "http://localhost:8080/";
   if (NULL == (twister_exchange_url = TALER_TWISTER_prepare_twister (
                  PROXY_EXCHANGE_config_file)))
