@@ -68,11 +68,22 @@ kill -TERM "$SETUP_PID"
 wait
 unset SETUP_PID
 
-setup -c test_template.conf -ef -u "exchange-account-2"
+setup -c test_template.conf \
+      -ef \
+      -u "exchange-account-2" \
+      -r "merchant-exchange-default"
 
 NEW_SECRET=secret-token:different_value
 
-taler-merchant-httpd -a "${NEW_SECRET}" -c "${CONF}" -L DEBUG 2> taler-merchant-httpd.log &
+taler-merchant-exchangekeyupdate \
+    -c "${CONF}" \
+    -L DEBUG \
+    2> taler-merchant-exchangekeyupdate2.log &
+taler-merchant-httpd \
+    -a "${NEW_SECRET}" \
+    -c "${CONF}" \
+    -L DEBUG \
+    2> taler-merchant-httpd2.log &
 # Install cleanup handler (except for kill -9)
 trap my_cleanup EXIT
 
@@ -84,7 +95,12 @@ do
     sleep 0.1
     OK=0
     # merchant
-    wget --waitretry=0 --timeout=1 http://localhost:9966/ -o /dev/null -O /dev/null >/dev/null || continue
+    wget --waitretry=0 \
+         --timeout=1 \
+         http://localhost:9966/ \
+         -o /dev/null \
+         -O /dev/null \
+         >/dev/null || continue
     OK=1
     break
 done
@@ -94,6 +110,7 @@ then
     exit_fail "Failed to (re)start merchant backend"
 fi
 
+
 echo -n "Creating order to test auth is ok..." >&2
 STATUS=$(curl -H "Content-Type: application/json" -X POST \
     'http://localhost:9966/private/orders' \
