@@ -34,6 +34,7 @@ echo " FOUND"
 
 # Launch exchange, merchant and bank.
 setup -c "test_template.conf" \
+      -r "merchant-exchange-default" \
       -em \
       $BANK_FLAGS
 LAST_RESPONSE=$(mktemp -p "${TMPDIR:-/tmp}" test_response.conf-XXXXXX)
@@ -90,6 +91,7 @@ fi
 # CREATE INSTANCE FOR TESTING
 #
 
+
 echo -n "Configuring merchant instance ..."
 
 STATUS=$(curl -H "Content-Type: application/json" -X POST \
@@ -134,9 +136,9 @@ then
     exit_fail "Expected '200 OK' response. Got instead $STATUS"
 fi
 
-
 echo "Ok"
 
+
 echo -n "Get accounts..."
 STATUS=$(curl http://localhost:9966/private/accounts \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
@@ -192,7 +194,7 @@ echo "OK"
 
 echo -n "Checking created order without TOKEN..."
 STATUS=$(curl http://localhost:9966/orders/"$ORDER_ID" \
-    -w "%{http_code}" -s -o "$LAST_RESPONSE")
+              -w "%{http_code}" -s -o "$LAST_RESPONSE")
 PAY_URI=$(jq -r .taler_pay_uri < "$LAST_RESPONSE")
 if [ "$PAY_URI" == "null" ]
 then
@@ -244,26 +246,45 @@ fi
 echo "OK"
 
 #
-# CREATE TOKEN FAMILY AND V1 ORDER WITH CHOICES
+# CREATE A DISCOUNT TOKEN FAMILY
 #
-echo -n "Creating token family ..."
-NOW=$(date +%s)
-IN_A_YEAR=$((NOW + 31536000))
+echo -n "Creating discount token family..."
+VALID_AFTER="{\"t_s\": $(date +%s)}" # now
+VALID_BEFORE="{\"t_s\": $(date +%s -d "+30 days")}" # 30 days from now
+DURATION="{\"d_us\": $(expr 30 \* 24 \* 60 \* 60 \* 1000000)}" # 30 days
 STATUS=$(curl 'http://localhost:9966/private/tokenfamilies' \
-    -d '{"slug":"test-sub","kind":"subscription","description":"Test token family","name":"Test Subscription","valid_after":{"t_s":'${NOW}'},"valid_before":{"t_s":'${IN_A_YEAR}'},"duration": {"d_us": 2592000000000},"rounding": {"d_us": 86400000000}}' \
-    -w "%{http_code}" -s -o "$LAST_RESPONSE")
-
+              -d "{\"kind\": \"discount\", \"slug\":\"test-discount\", \"name\": \"Test discount\", \"description\": \"Less money $$\", \"description_i18n\": {\"en\": \"Less money $$\", \"es\": \"Menos dinero $$\"}, \"valid_after\": $VALID_AFTER, \"valid_before\": $VALID_BEFORE, \"duration\": $DURATION, \"validity_granularity\": $DURATION}" \
+              -w "%{http_code}" -s -o /dev/null)
 if [ "$STATUS" != "204" ]
 then
-    cat "$LAST_RESPONSE" >&2
-    exit_fail "Expected 204, token family created. got: $STATUS"
+    exit_fail "Expected '204 OK' response. Got instead $STATUS"
 fi
+echo "Ok"
 
-echo " OK"
+#
+# CREATE A SUBSCRIPTION TOKEN FAMILY
+#
+echo -n "Creating subscription token family..."
+VALID_AFTER="{\"t_s\": $(date +%s)}" # now
+VALID_BEFORE="{\"t_s\": $(date +%s -d "+30 days")}" # 30 days from now
+DURATION="{\"d_us\": $(expr 30 \* 24 \* 60 \* 60 \* 1000000)}" # 30 days
+STATUS=$(curl 'http://localhost:9966/private/tokenfamilies' \
+              -d "{\"kind\": \"subscription\", \"slug\":\"test-subscription\", \"name\": \"Test subscription\", \"description\": \"Money per month\", \"description_i18n\": {\"en\": \"Money $$$ per month\", \"es\": \"Dinero $$$ al mes\"}, \"valid_after\": $VALID_AFTER, \"valid_before\": $VALID_BEFORE, \"duration\": $DURATION, \"validity_granularity\": $DURATION}" \
+              -w "%{http_code}" -s -o /dev/null)
+if [ "$STATUS" != "204" ]
+then
+    exit_fail "Expected '204 OK' response. Got instead $STATUS"
+fi
+echo "Ok"
 
-echo -n "Creating v1 order with token family ..."
+#
+# CREATE AN DISCOUNTABLE ORDER WITHOUT TOKEN
+#
+RANDOM_IMG='data:image/png;base64,abcdefg'
+
+echo -n "Creating discountable order..."
 STATUS=$(curl 'http://localhost:9966/private/orders' \
-    -d '{"order":{"version":"1","amount":"TESTKUDOS:7","summary":"with_subscription","fulfillment_message":"Paid successfully","choices":[{"inputs":[{"kind":"token","count":1,"token_family_slug":"test-sub","valid_after":{"t_s":'$NOW'}}],"outputs":[{"kind":"token","count":1,"token_family_slug":"test-sub","valid_after":{"t_s":'$NOW'}}]}]}}' \
+    -d '{"create_token":true,"order":{"version":1,"summary":"Expensive purchase","products":[{"description":"Expensive steak","quantity":2,"unit":"pieces","price":"TESTKUDOS:100"}],"choices":[{"amount":"TESTKUDOS:100"},{"amount":"TESTKUDOS:10","inputs":[{"type":"token","token_family_slug":"test-discount","count":1}],"outputs":[]}]}}' \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
 
 if [ "$STATUS" != "200" ]
@@ -271,14 +292,23 @@ then
     cat "$LAST_RESPONSE" >&2
     exit_fail "Expected 200, order created. got: $STATUS"
 fi
-
-echo " OK"
-
-echo -n "Claming order with token family ..."
+echo "OK"
 
 ORDER_ID=$(jq -r .order_id < "$LAST_RESPONSE")
 TOKEN=$(jq -r .token < "$LAST_RESPONSE")
 
+echo -n "Checking created order..."
+STATUS=$(curl http://localhost:9966/orders/"$ORDER_ID?token=$TOKEN" \
+              -w "%{http_code}" -s -o "$LAST_RESPONSE")
+PAY_URI=$(jq -r .taler_pay_uri < "$LAST_RESPONSE")
+if [ "$PAY_URI" == "null" ]
+then
+    cat "$LAST_RESPONSE" >&2
+    exit_fail "Expected non-NULL payuri. got $PAY_URI"
+fi
+echo "OK"
+
+echo -n "Claming order with token family ..."
 STATUS=$(curl http://localhost:9966/orders/"$ORDER_ID"/claim \
     -d '{"nonce":"","token":"'"$TOKEN"'"}' \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
@@ -601,9 +631,9 @@ then
 fi
 
 echo " OK"
-echo -n "Testing taler-merchant-exchange ..."
+echo -n "Testing taler-merchant-reconciliation ..."
 set -e
-taler-merchant-exchange -L INFO -c "$CONF" -t &> taler-merchant-exchange.log
+taler-merchant-reconciliation -L INFO -c "$CONF" -t &> taler-merchant-reconciliation.log
 echo " OK"
 
 
@@ -685,9 +715,9 @@ echo -n "Getting information about kyc ..."
 STATUS=$(curl -H "Content-Type: application/json" -X GET \
     http://localhost:9966/private/kyc \
     -w "%{http_code}" -s -o /dev/null)
-if [ "$STATUS" != "204" ]
+if [ "$STATUS" != "200" ]
 then
-    exit_fail "Expected 204. Got: $STATUS"
+    exit_fail "Expected 200. Got: $STATUS"
 fi
 echo " OK"
 
