@@ -47,6 +47,7 @@ echo " FOUND"
 
 # Launch system.
 setup -c "test_template.conf" \
+      -r "merchant-exchange-default" \
       -em \
       $BANK_FLAGS
 LAST_RESPONSE=$(mktemp -p "${TMPDIR:-/tmp}" test_response.conf-XXXXXX)
@@ -307,8 +308,8 @@ fi
 
 echo "OK"
 
-echo -n "Fetching running taler-merchant-exchange on bogus transfer ..."
-taler-merchant-exchange -c "$CONF" -L INFO -t &> taler-merchant-exchange-bad.log
+echo -n "Fetching running taler-merchant-reconciliation on bogus transfer ..."
+taler-merchant-reconciliation -c "$CONF" -L INFO -t &> taler-merchant-reconciliation-bad.log
 echo "OK"
 
 echo -n "Fetching wire transfers of 'test' instance ..."
@@ -367,8 +368,8 @@ then
 fi
 echo " OK"
 
-echo -n "Fetching running taler-merchant-exchange on good transfer ..."
-taler-merchant-exchange -c $CONF -L INFO -t &> taler-merchant-exchange-bad.log
+echo -n "Fetching running taler-merchant-reconciliation on good transfer ..."
+taler-merchant-reconciliation -c $CONF -L INFO -t &> taler-merchant-reconciliation-bad.log
 echo "OK"
 
 echo -n "Fetching wire transfers of TEST instance ..."
@@ -537,8 +538,8 @@ then
 fi
 echo " OK"
 
-echo -n "Fetching running taler-merchant-exchange on good transfer ..."
-taler-merchant-exchange -c $CONF -L INFO -t &> taler-merchant-exchange2.log
+echo -n "Fetching running taler-merchant-reconciliation on good transfer ..."
+taler-merchant-reconciliation -c $CONF -L INFO -t &> taler-merchant-reconciliation2.log
 echo "OK"
 
 echo -n "Fetching wire transfers of TEST instance ..."
@@ -684,8 +685,8 @@ fi
 
 echo "OK"
 
-echo -n "Fetching running taler-merchant-exchange on good transfer ..."
-taler-merchant-exchange -c $CONF -L INFO -t &> taler-merchant-exchange2.log
+echo -n "Fetching running taler-merchant-reconciliation on good transfer ..."
+taler-merchant-reconciliation -c $CONF -L INFO -t &> taler-merchant-reconciliation2.log
 echo "OK"
 
 echo -n "Checking order status ..."
