@@ -53,6 +53,7 @@ fi
 # Launch exchange, merchant and bank.
 setup -c "test_template.conf" \
       -em \
+      -r "merchant-exchange-default" \
       $BANK_FLAGS
 LAST_RESPONSE=$(mktemp -p "${TMPDIR:-/tmp}" test_response.conf-XXXXXX)
 CONF="test_template.conf.edited"
@@ -64,7 +65,7 @@ if [ 1 = "$USE_FAKEBANK" ]
 then
     FACADE_URL="http://localhost:8082/accounts/gnunet/taler-revenue/"
     FACADE_USERNAME="gnunet"
-    FACADE_PASSWORD="x"
+    FACADE_PASSWORD="password"
 else
     echo "not implemented for current libeufin-bank"
     exit 1
@@ -347,10 +348,10 @@ fi
 echo " OK"
 
 echo -n "Integrating wire transfer data with exchange..."
-taler-merchant-exchange \
+taler-merchant-reconciliation \
     -c "$CONF" \
     -t \
-    -L INFO &> merchant-exchange.log
+    -L INFO &> merchant-reconciliation.log
 echo " OK"
 
 echo -n "Checking order status ..."
