@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2018, 2020 Taler Systems SA
+  Copyright (C) 2014-2024 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as
@@ -144,18 +144,32 @@ struct PayState
  *
  * @param token_families json object of token families where the key is the slug
  * @param slug the slug of the token family
- * @param valid_after the timestamp of the token family
+ * @param key_index index of the key within the token family
  * @param[out] pub the token issue public key of the token family
  * @return #GNUNET_OK on success and #GNUNET_SYSERR if not found
  */
 static enum GNUNET_GenericReturnValue
 find_token_public_key (const json_t *token_families,
                        const char *slug,
-                       struct GNUNET_TIME_Timestamp valid_after,
-                       struct TALER_TokenIssuePublicKeyP *pub)
+                       unsigned int key_index,
+                       struct TALER_TokenIssuePublicKey *pub)
+
 {
   const json_t *tf = json_object_get (token_families, slug);
   const json_t *keys;
+  struct GNUNET_JSON_Specification spec[] = {
+    GNUNET_JSON_spec_array_const ("keys",
+                                  &keys),
+    GNUNET_JSON_spec_end ()
+  };
+  const json_t *key;
+  const char *error_name;
+  unsigned int error_line;
+  struct GNUNET_JSON_Specification ispec[] = {
+    TALER_JSON_spec_token_pub (NULL,
+                               pub),
+    GNUNET_JSON_spec_end ()
+  };
 
   if (NULL == tf)
   {
@@ -164,13 +178,6 @@ find_token_public_key (const json_t *token_families,
                 slug);
     return GNUNET_SYSERR;
   }
-
-  struct GNUNET_JSON_Specification spec[] = {
-    GNUNET_JSON_spec_array_const ("keys",
-                                  &keys),
-    GNUNET_JSON_spec_end ()
-  };
-
   if (GNUNET_OK !=
       GNUNET_JSON_parse (tf,
                          spec,
@@ -183,75 +190,30 @@ find_token_public_key (const json_t *token_families,
     return GNUNET_SYSERR;
   }
 
+  key = json_array_get (keys,
+                        key_index);
+  if (NULL == key)
   {
-    unsigned int i;
-    const json_t *key;
-
-    json_array_foreach (keys, i, key)
-    {
-      int64_t cipher;
-      struct GNUNET_TIME_Timestamp ivalid_after;
-      struct GNUNET_CRYPTO_BlindSignPublicKey *issue_pub = GNUNET_new (struct GNUNET_CRYPTO_BlindSignPublicKey);
-      const char *error_name;
-      unsigned int error_line;
-      struct GNUNET_JSON_Specification ispec[] = {
-        GNUNET_JSON_spec_fixed_auto ("h_pub",
-                                     &issue_pub->pub_key_hash),
-        GNUNET_JSON_spec_rsa_public_key ("rsa_pub",
-                                         &issue_pub->details.rsa_public_key),
-        //   GNUNET_JSON_spec_fixed_auto ("cs_pub",
-        //                               &key.pub.public_key->details.cs_public_key)),
-        GNUNET_JSON_spec_int64 ("cipher",
-                                &cipher),
-        GNUNET_JSON_spec_timestamp ("valid_after",
-                                    &ivalid_after),
-        GNUNET_JSON_spec_end()
-      };
-
-      if (GNUNET_OK !=
-          GNUNET_JSON_parse (key,
-                             ispec,
-                             &error_name,
-                             &error_line))
-      {
-        GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                    "Failed to parse %s at %u: %s\n",
-                    ispec[error_line].field,
-                    error_line,
-                    error_name);
-        return GNUNET_SYSERR;
-      }
-
-      switch (cipher) {
-      case GNUNET_CRYPTO_BSA_RSA:
-        issue_pub->cipher = GNUNET_CRYPTO_BSA_RSA;
-        break;
-      case GNUNET_CRYPTO_BSA_CS:
-        issue_pub->cipher = GNUNET_CRYPTO_BSA_CS;
-        break;
-      case GNUNET_CRYPTO_BSA_INVALID:
-        GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                    "Field 'cipher' invalid in key #%u\n",
-                    i);
-        return GNUNET_SYSERR;
-      }
-
-      /* Compare valid_after to make sure it matches. */
-      if (GNUNET_TIME_timestamp_cmp(valid_after, !=, ivalid_after))
-      {
-        continue;
-      }
-
-      pub->public_key = issue_pub;
-      return GNUNET_OK;
-    }
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Key with index %u for token family '%s' not found\n",
+                key_index,
+                slug);
+    return GNUNET_SYSERR;
   }
-
-  GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-              "Key with valid_after '%s' for token family '%s' not found\n",
-              GNUNET_TIME_timestamp2s(valid_after),
-              slug);
-  return GNUNET_SYSERR;
+  if (GNUNET_OK !=
+      GNUNET_JSON_parse (key,
+                         ispec,
+                         &error_name,
+                         &error_line))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Failed to parse %s at %u: %s\n",
+                ispec[error_line].field,
+                error_line,
+                error_name);
+    return GNUNET_SYSERR;
+  }
+  return GNUNET_OK;
 }
 
 
@@ -300,7 +262,7 @@ build_coins (struct TALER_MERCHANT_PayCoin **pc,
 
     /* Token syntax is "LABEL[/NUMBER]" */
     ctok = strchr (token, '/');
-    // TODO: Check why ci variable is parsed but not used?
+    /* FIXME: Check why ci variable is parsed but not used? */
     ci = 0;
     if (NULL != ctok)
     {
@@ -355,7 +317,8 @@ build_coins (struct TALER_MERCHANT_PayCoin **pc,
       GNUNET_assert (GNUNET_OK ==
                      TALER_TESTING_get_trait_h_age_commitment (coin_cmd,
                                                                0,
-                                                               &h_age_commitment));
+                                                               &h_age_commitment
+                                                               ));
       icoin->coin_priv = *coin_priv;
       icoin->denom_pub = denom_pub->key;
       icoin->denom_sig = *denom_sig;
@@ -442,8 +405,8 @@ build_tokens (struct TALER_MERCHANT_UseToken **tokens,
 
     {
       const struct TALER_TokenUsePrivateKeyP *token_priv;
-      const struct TALER_TokenIssueSignatureP *issue_sig;
-      const struct TALER_TokenIssuePublicKeyP *issue_pub;
+      const struct TALER_TokenIssueSignature *issue_sig;
+      const struct TALER_TokenIssuePublicKey *issue_pub;
 
       GNUNET_assert (GNUNET_OK ==
                      TALER_TESTING_get_trait_token_priv (pay_cmd,
@@ -469,6 +432,7 @@ build_tokens (struct TALER_MERCHANT_UseToken **tokens,
   return GNUNET_OK;
 }
 
+
 /**
  * Function called with the result of a /pay operation.
  * Checks whether the merchant signature is valid and the
@@ -596,8 +560,6 @@ pay_run (void *cls,
   struct TALER_MerchantWireHashP h_wire;
   const struct TALER_PrivateContractHashP *h_proposal;
   struct TALER_Amount max_fee;
-  const json_t *choices = NULL;
-  const json_t *token_families = NULL;
   const char *error_name = NULL;
   unsigned int error_line = 0;
   struct TALER_MERCHANT_PayCoin *pay_coins;
@@ -635,7 +597,12 @@ pay_run (void *cls,
   {
     /* Get information that needs to be put verbatim in the
      * deposit permission */
+    uint64_t version = 0;
     struct GNUNET_JSON_Specification spec[] = {
+      GNUNET_JSON_spec_mark_optional (
+        GNUNET_JSON_spec_uint64 ("version",
+                                 &version),
+        NULL),
       GNUNET_JSON_spec_string ("order_id",
                                &order_id),
       GNUNET_JSON_spec_timestamp ("refund_deadline",
@@ -648,18 +615,6 @@ pay_run (void *cls,
                                    &merchant_pub),
       GNUNET_JSON_spec_fixed_auto ("h_wire",
                                    &h_wire),
-      TALER_JSON_spec_amount_any ("amount",
-                                  &ps->total_amount),
-      TALER_JSON_spec_amount_any ("max_fee",
-                                  &max_fee),
-      GNUNET_JSON_spec_mark_optional (
-        GNUNET_JSON_spec_object_const ("token_families",
-                                      &token_families),
-        NULL),
-      GNUNET_JSON_spec_mark_optional (
-        GNUNET_JSON_spec_array_const ("choices",
-                                      &choices),
-        NULL),
       /* FIXME oec: parse minimum age, use data later? */
       GNUNET_JSON_spec_end ()
     };
@@ -682,6 +637,221 @@ pay_run (void *cls,
       free (js);
       TALER_TESTING_FAIL (is);
     }
+    switch (version)
+    {
+    case 0:
+      {
+        struct GNUNET_JSON_Specification v0spec[] = {
+          TALER_JSON_spec_amount_any ("amount",
+                                      &ps->total_amount),
+          TALER_JSON_spec_amount_any ("max_fee",
+                                      &max_fee),
+          GNUNET_JSON_spec_end ()
+        };
+
+        if (GNUNET_OK !=
+            GNUNET_JSON_parse (contract_terms,
+                               v0spec,
+                               &error_name,
+                               &error_line))
+        {
+          char *js;
+
+          js = json_dumps (contract_terms,
+                           JSON_INDENT (1));
+          GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                      "Parser failed on %s:%u for input `%s'\n",
+                      error_name,
+                      error_line,
+                      js);
+          free (js);
+          TALER_TESTING_FAIL (is);
+        }
+      }
+      if (0 < ps->choice_index)
+        TALER_TESTING_FAIL (is);
+      break;
+    case 1:
+      {
+        const json_t *choices;
+        const json_t *token_families;
+        struct GNUNET_JSON_Specification v1spec[] = {
+          GNUNET_JSON_spec_object_const ("token_families",
+                                         &token_families),
+          GNUNET_JSON_spec_array_const ("choices",
+                                        &choices),
+          GNUNET_JSON_spec_end ()
+        };
+        const json_t *outputs;
+        json_t *output;
+        unsigned int output_index;
+        const json_t *choice;
+
+        if (GNUNET_OK !=
+            GNUNET_JSON_parse (contract_terms,
+                               v1spec,
+                               &error_name,
+                               &error_line))
+        {
+          char *js;
+
+          js = json_dumps (contract_terms,
+                           JSON_INDENT (1));
+          GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                      "Parser failed on %s:%u for input `%s'\n",
+                      error_name,
+                      error_line,
+                      js);
+          free (js);
+          TALER_TESTING_FAIL (is);
+        }
+
+        choice = json_array_get (choices,
+                                 ps->choice_index);
+        if (NULL == choice)
+        {
+          GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                      "No choice found at index %d\n",
+                      ps->choice_index);
+          TALER_TESTING_FAIL (is);
+        }
+
+        {
+          const char *ierror_name = NULL;
+          unsigned int ierror_line = 0;
+
+          struct GNUNET_JSON_Specification ispec[] = {
+            TALER_JSON_spec_amount_any ("amount",
+                                        &ps->total_amount),
+            TALER_JSON_spec_amount_any ("max_fee",
+                                        &max_fee),
+            GNUNET_JSON_spec_array_const ("outputs",
+                                          &outputs),
+            GNUNET_JSON_spec_end ()
+          };
+
+          if (GNUNET_OK !=
+              GNUNET_JSON_parse (choice,
+                                 ispec,
+                                 &ierror_name,
+                                 &ierror_line))
+          {
+            GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                        "Parser failed on %s:%u for input `%s'\n",
+                        ierror_name,
+                        ierror_line,
+                        json_dumps (choice,
+                                    JSON_INDENT (2)));
+            TALER_TESTING_FAIL (is);
+          }
+        }
+
+        json_array_foreach (outputs, output_index, output)
+        {
+          const char *slug;
+          const char *kind;
+          uint32_t key_index;
+          uint32_t count = 1;
+          const char *ierror_name = NULL;
+          unsigned int ierror_line = 0;
+
+          struct GNUNET_JSON_Specification ispec[] = {
+            GNUNET_JSON_spec_string ("type",
+                                     &kind),
+            GNUNET_JSON_spec_string ("token_family_slug",
+                                     &slug),
+            GNUNET_JSON_spec_uint32 ("key_index",
+                                     &key_index),
+            GNUNET_JSON_spec_mark_optional (
+              GNUNET_JSON_spec_uint32 ("count",
+                                       &count),
+              NULL),
+            GNUNET_JSON_spec_end ()
+          };
+
+          if (GNUNET_OK !=
+              GNUNET_JSON_parse (output,
+                                 ispec,
+                                 &ierror_name,
+                                 &ierror_line))
+          {
+            GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                        "Parser failed on %s:%u for input `%s'\n",
+                        ierror_name,
+                        ierror_line,
+                        json_dumps (output,
+                                    JSON_INDENT (2)));
+            TALER_TESTING_FAIL (is);
+          }
+
+          if (0 != strcmp ("token", kind))
+          {
+            continue;
+          }
+
+          GNUNET_array_grow (ps->issued_tokens,
+                             ps->num_issued_tokens,
+                             ps->num_issued_tokens + count);
+
+          for (unsigned int k = 0; k < count; k++)
+          {
+            struct TALER_MERCHANT_PrivateTokenDetails *details =
+              &ps->issued_tokens[ps->num_issued_tokens - count + k];
+
+            if (GNUNET_OK !=
+                find_token_public_key (token_families,
+                                       slug,
+                                       key_index,
+                                       &details->issue_pub))
+            {
+              TALER_TESTING_FAIL (is);
+            }
+
+            /* Only RSA is supported for now. */
+            GNUNET_assert (GNUNET_CRYPTO_BSA_RSA ==
+                           details->issue_pub.public_key->cipher);
+
+            TALER_token_blind_input_copy (&details->blinding_inputs,
+                                          TALER_token_blind_input_rsa_singleton ()
+                                          );
+            /* FIXME: Where to get details->blinding_inputs from? */
+            TALER_token_use_setup_random (&details->master);
+            TALER_token_use_setup_priv (&details->master,
+                                        &details->blinding_inputs,
+                                        &details->token_priv);
+            TALER_token_use_blinding_secret_create (&details->master,
+                                                    &details->blinding_inputs,
+                                                    &details->blinding_secret);
+            GNUNET_CRYPTO_eddsa_key_get_public (&details->token_priv.private_key
+                                                ,
+                                                &details->token_pub.public_key);
+            GNUNET_CRYPTO_hash (&details->token_pub.public_key,
+                                sizeof (struct GNUNET_CRYPTO_EcdsaPublicKey),
+                                &details->h_token_pub.hash);
+            details->envelope.blinded_pub = GNUNET_CRYPTO_message_blind_to_sign
+                                            (
+              details->issue_pub.public_key,
+              &details->blinding_secret,
+              NULL, /* FIXME: Add session nonce to support CS tokens */
+              &details->h_token_pub.hash,
+              sizeof (details->h_token_pub.hash),
+              details->blinding_inputs.blinding_inputs);
+
+            if (NULL == details->envelope.blinded_pub)
+            {
+              GNUNET_break (0);
+              TALER_TESTING_FAIL (is);
+            }
+          }
+        }
+      }
+
+      break;
+    default:
+      TALER_TESTING_FAIL (is);
+    }
+
+
   }
 
   {
@@ -725,148 +895,14 @@ pay_run (void *cls,
     }
     GNUNET_free (tr);
   }
-  if (0 <= ps->choice_index)
-  {
-    const json_t *outputs;
-    json_t *output;
-    unsigned int output_index;
-    const json_t *choice = json_array_get (choices, ps->choice_index);
-
-    if (NULL == choice)
-    {
-      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                  "No choice found at index %d\n",
-                  ps->choice_index);
-      TALER_TESTING_FAIL (is);
-    }
-
-    {
-      const char *ierror_name = NULL;
-      unsigned int ierror_line = 0;
-
-      struct GNUNET_JSON_Specification ispec[] = {
-        GNUNET_JSON_spec_array_const ("outputs",
-                                      &outputs),
-        GNUNET_JSON_spec_end ()
-      };
-
-      if (GNUNET_OK !=
-          GNUNET_JSON_parse (choice,
-                             ispec,
-                             &ierror_name,
-                             &ierror_line))
-      {
-        GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                    "Parser failed on %s:%u for input `%s'\n",
-                    ierror_name,
-                    ierror_line,
-                    json_dumps (choice,
-                                JSON_INDENT (2)));
-        TALER_TESTING_FAIL (is);
-      }
-    }
-
-    json_array_foreach (outputs, output_index, output)
-    {
-      const char *slug;
-      const char *kind;
-      struct GNUNET_TIME_Timestamp valid_after;
-      uint32_t count = 1;
-      const char *ierror_name = NULL;
-      unsigned int ierror_line = 0;
-
-      struct GNUNET_JSON_Specification ispec[] = {
-        GNUNET_JSON_spec_string ("kind",
-                                  &kind),
-        GNUNET_JSON_spec_string ("token_family_slug",
-                                  &slug),
-        GNUNET_JSON_spec_timestamp ("valid_after",
-                                    &valid_after),
-        GNUNET_JSON_spec_mark_optional (
-          GNUNET_JSON_spec_uint32 ("count",
-                                    &count),
-          NULL),
-        GNUNET_JSON_spec_end()
-      };
-
-      if (GNUNET_OK !=
-          GNUNET_JSON_parse (output,
-                            ispec,
-                            &ierror_name,
-                            &ierror_line))
-      {
-        GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                    "Parser failed on %s:%u for input `%s'\n",
-                    ierror_name,
-                    ierror_line,
-                    json_dumps (output,
-                                JSON_INDENT (2)));
-        TALER_TESTING_FAIL (is);
-      }
-
-      if (0 != strcmp("token", kind))
-      {
-        continue;
-      }
-
-      GNUNET_array_grow (ps->issued_tokens,
-                         ps->num_issued_tokens,
-                         ps->num_issued_tokens + count);
-
-      for (unsigned int k = 0; k < count; k++)
-      {
-        struct TALER_MERCHANT_PrivateTokenDetails *details =
-              &ps->issued_tokens[ps->num_issued_tokens - count + k];
-
-        if (GNUNET_OK != find_token_public_key (token_families,
-                                                slug,
-                                                valid_after,
-                                                &details->issue_pub))
-        {
-          TALER_TESTING_FAIL (is);
-        }
-
-        /* Only RSA is supported for now. */
-        GNUNET_assert (GNUNET_CRYPTO_BSA_RSA == details->issue_pub.public_key->cipher);
-
-        TALER_token_blind_input_copy (&details->blinding_inputs,
-                                      TALER_token_blind_input_rsa_singleton ());
-        /* TODO: Where to get details->blinding_inputs from? */
-        TALER_token_use_setup_random (&details->master);
-        TALER_token_use_setup_priv (&details->master,
-                                    &details->blinding_inputs,
-                                    &details->token_priv);
-        TALER_token_use_blinding_secret_create (&details->master,
-                                                &details->blinding_inputs,
-                                                &details->blinding_secret);
-        GNUNET_CRYPTO_eddsa_key_get_public (&details->token_priv.private_key,
-                                            &details->token_pub.public_key);
-        GNUNET_CRYPTO_hash (&details->token_pub.public_key,
-                            sizeof (struct GNUNET_CRYPTO_EcdsaPublicKey),
-                            &details->h_token_pub.hash);
-        details->envelope.blinded_pub = GNUNET_CRYPTO_message_blind_to_sign (
-          details->issue_pub.public_key,
-          &details->blinding_secret,
-          NULL, /* TODO: Add session nonce to support CS tokens */
-          &details->h_token_pub.hash,
-          sizeof (details->h_token_pub.hash),
-          details->blinding_inputs.blinding_inputs);
-
-        if (NULL == details->envelope.blinded_pub)
-        {
-          GNUNET_break (0);
-          TALER_TESTING_FAIL (is);
-        }
-      }
-    }
-  }
 
   GNUNET_array_grow (output_tokens,
                      len_output_tokens,
                      ps->num_issued_tokens);
   for (unsigned int i = 0; i<len_output_tokens; i++)
   {
-    output_tokens[i].envelope.blinded_pub = ps->issued_tokens[i].envelope.blinded_pub;
+    output_tokens[i].envelope.blinded_pub = ps->issued_tokens[i].envelope.
+                                            blinded_pub;
   }
 
   if (GNUNET_OK !=
@@ -879,29 +915,29 @@ pay_run (void *cls,
                                                 &h_proposal))
     TALER_TESTING_FAIL (is);
   ps->h_contract_terms = *h_proposal;
-  ps->oph = TALER_MERCHANT_order_pay (TALER_TESTING_interpreter_get_context (
-                                        is),
-                                      ps->merchant_url,
-                                      ps->session_id,
-                                      h_proposal,
-                                      ps->choice_index,
-                                      &ps->total_amount,
-                                      &max_fee,
-                                      &merchant_pub,
-                                      merchant_sig,
-                                      timestamp,
-                                      refund_deadline,
-                                      pay_deadline,
-                                      &h_wire,
-                                      order_id,
-                                      npay_coins,
-                                      pay_coins,
-                                      len_use_tokens,
-                                      use_tokens,
-                                      len_output_tokens,
-                                      output_tokens,
-                                      &pay_cb,
-                                      ps);
+  ps->oph = TALER_MERCHANT_order_pay (
+    TALER_TESTING_interpreter_get_context (is),
+    ps->merchant_url,
+    ps->session_id,
+    h_proposal,
+    ps->choice_index,
+    &ps->total_amount,
+    &max_fee,
+    &merchant_pub,
+    merchant_sig,
+    timestamp,
+    refund_deadline,
+    pay_deadline,
+    &h_wire,
+    order_id,
+    npay_coins,
+    pay_coins,
+    len_use_tokens,
+    use_tokens,
+    len_output_tokens,
+    output_tokens,
+    &pay_cb,
+    ps);
   GNUNET_array_grow (pay_coins,
                      npay_coins,
                      0);
@@ -1005,11 +1041,14 @@ pay_traits (void *cls,
         TALER_TESTING_make_trait_otp_key (ps->pos_key),
         TALER_TESTING_make_trait_otp_alg (&ps->pos_alg),
         TALER_TESTING_make_trait_token_priv (index,
-                                             &ps->issued_tokens[index].token_priv),
+                                             &ps->issued_tokens[index].
+                                             token_priv),
         TALER_TESTING_make_trait_token_issue_pub (index,
-                                                  &ps->issued_tokens[index].issue_pub),
+                                                  &ps->issued_tokens[index].
+                                                  issue_pub),
         TALER_TESTING_make_trait_token_issue_sig (index,
-                                                  &ps->issued_tokens[index].issue_sig),
+                                                  &ps->issued_tokens[index].
+                                                  issue_sig),
         TALER_TESTING_trait_end ()
       };
 
